@@ -1,27 +1,81 @@
-from locale import getlocale
+import locale as stdlib_locale
 from logging import getLogger
 
 from cached_property import threaded_cached_property
 
-from .autodiscover import discover
+from .autodiscover import Autodiscovery
 from .configuration import Configuration
-from .credentials import DELEGATE, IMPERSONATION, ACCESS_TYPES
-from .errors import UnknownTimeZone
-from .ewsdatetime import EWSTimeZone, UTC
+from .credentials import ACCESS_TYPES, DELEGATE, IMPERSONATION
+from .errors import InvalidEnumValue, InvalidTypeError, UnknownTimeZone
+from .ewsdatetime import UTC, EWSTimeZone
 from .fields import FieldPath
-from .folders import Folder, AdminAuditLogs, ArchiveDeletedItems, ArchiveInbox, ArchiveMsgFolderRoot, \
-    ArchiveRecoverableItemsDeletions, ArchiveRecoverableItemsPurges, ArchiveRecoverableItemsRoot, \
-    ArchiveRecoverableItemsVersions, ArchiveRoot, Calendar, Conflicts, Contacts, ConversationHistory, DeletedItems, \
-    Directory, Drafts, Favorites, IMContactList, Inbox, Journal, JunkEmail, LocalFailures, MsgFolderRoot, MyContacts, \
-    Notes, Outbox, PeopleConnect, PublicFoldersRoot, QuickContacts, RecipientCache, RecoverableItemsDeletions, \
-    RecoverableItemsPurges, RecoverableItemsRoot, RecoverableItemsVersions, Root, SearchFolders, SentItems, \
-    ServerFailures, SyncIssues, Tasks, ToDoSearch, VoiceMail
-from .items import HARD_DELETE, AUTO_RESOLVE, SEND_TO_NONE, SAVE_ONLY, ALL_OCCURRENCIES, ID_ONLY
+from .folders import (
+    AdminAuditLogs,
+    ArchiveDeletedItems,
+    ArchiveInbox,
+    ArchiveMsgFolderRoot,
+    ArchiveRecoverableItemsDeletions,
+    ArchiveRecoverableItemsPurges,
+    ArchiveRecoverableItemsRoot,
+    ArchiveRecoverableItemsVersions,
+    ArchiveRoot,
+    Calendar,
+    Conflicts,
+    Contacts,
+    ConversationHistory,
+    DeletedItems,
+    Directory,
+    Drafts,
+    Favorites,
+    Folder,
+    IMContactList,
+    Inbox,
+    Journal,
+    JunkEmail,
+    LocalFailures,
+    MsgFolderRoot,
+    MyContacts,
+    Notes,
+    Outbox,
+    PeopleConnect,
+    PublicFoldersRoot,
+    QuickContacts,
+    RecipientCache,
+    RecoverableItemsDeletions,
+    RecoverableItemsPurges,
+    RecoverableItemsRoot,
+    RecoverableItemsVersions,
+    Root,
+    SearchFolders,
+    SentItems,
+    ServerFailures,
+    SyncIssues,
+    Tasks,
+    ToDoSearch,
+    VoiceMail,
+)
+from .items import ALL_OCCURRENCES, AUTO_RESOLVE, HARD_DELETE, ID_ONLY, SAVE_ONLY, SEND_TO_NONE
 from .properties import Mailbox, SendingAs
 from .protocol import Protocol
 from .queryset import QuerySet
-from .services import ExportItems, UploadItems, GetItem, CreateItem, UpdateItem, DeleteItem, MoveItem, SendItem, \
-    CopyItem, GetUserOofSettings, SetUserOofSettings, GetMailTips, ArchiveItem, GetDelegate, MarkAsJunk, GetPersona
+from .services import (
+    ArchiveItem,
+    CopyItem,
+    CreateItem,
+    DeleteItem,
+    ExportItems,
+    GetDelegate,
+    GetItem,
+    GetMailTips,
+    GetPersona,
+    GetUserOofSettings,
+    MarkAsJunk,
+    MoveItem,
+    SendItem,
+    SetUserOofSettings,
+    UpdateItem,
+    UploadItems,
+)
 from .util import get_domain, peek
 
 log = getLogger(__name__)
@@ -45,10 +99,7 @@ class Identity:
         self.sid = sid
 
     def __eq__(self, other):
-        for k in self.__dict__:
-            if getattr(self, k) != getattr(other, k):
-                return False
-        return True
+        return all(getattr(self, k) == getattr(other, k) for k in self.__dict__)
 
     def __hash__(self):
         return hash(repr(self))
@@ -60,8 +111,17 @@ class Identity:
 class Account:
     """Models an Exchange server user account."""
 
-    def __init__(self, primary_smtp_address, fullname=None, access_type=None, autodiscover=False, credentials=None,
-                 config=None, locale=None, default_timezone=None):
+    def __init__(
+        self,
+        primary_smtp_address,
+        fullname=None,
+        access_type=None,
+        autodiscover=False,
+        credentials=None,
+        config=None,
+        locale=None,
+        default_timezone=None,
+    ):
         """
 
         :param primary_smtp_address: The primary email address associated with the account on the Exchange server
@@ -78,60 +138,71 @@ class Account:
             assume values to be in the provided timezone. Defaults to the timezone of the host.
         :return:
         """
-        if '@' not in primary_smtp_address:
-            raise ValueError("primary_smtp_address %r is not an email address" % primary_smtp_address)
+        if "@" not in primary_smtp_address:
+            raise ValueError(f"primary_smtp_address {primary_smtp_address!r} is not an email address")
         self.fullname = fullname
         # Assume delegate access if individual credentials are provided. Else, assume service user with impersonation
         self.access_type = access_type or (DELEGATE if credentials else IMPERSONATION)
         if self.access_type not in ACCESS_TYPES:
-            raise ValueError("'access_type' %r must be one of %s" % (self.access_type, ACCESS_TYPES))
+            raise InvalidEnumValue("access_type", self.access_type, ACCESS_TYPES)
         try:
-            self.locale = locale or getlocale()[0] or None  # get_locale() might not be able to determine the locale
+            # get_locale() might not be able to determine the locale
+            self.locale = locale or stdlib_locale.getlocale()[0] or None
         except ValueError as e:
             # getlocale() may throw ValueError if it fails to parse the system locale
-            log.warning('Failed to get locale (%s)', e)
+            log.warning("Failed to get locale (%s)", e)
             self.locale = None
         if not isinstance(self.locale, (type(None), str)):
-            raise ValueError("Expected 'locale' to be a string, got %r" % self.locale)
+            raise InvalidTypeError("locale", self.locale, str)
         if default_timezone:
             try:
                 self.default_timezone = EWSTimeZone.from_timezone(default_timezone)
             except TypeError:
-                raise ValueError("Expected 'default_timezone' to be an EWSTimeZone, got %r" % default_timezone)
+                raise InvalidTypeError("default_timezone", default_timezone, EWSTimeZone)
         else:
             try:
                 self.default_timezone = EWSTimeZone.localzone()
             except (ValueError, UnknownTimeZone) as e:
                 # There is no translation from local timezone name to Windows timezone name, or e failed to find the
                 # local timezone.
-                log.warning('%s. Fallback to UTC', e.args[0])
+                log.warning("%s. Fallback to UTC", e.args[0])
                 self.default_timezone = UTC
         if not isinstance(config, (Configuration, type(None))):
-            raise ValueError("Expected 'config' to be a Configuration, got %r" % config)
+            raise InvalidTypeError("config", config, Configuration)
         if autodiscover:
             if config:
-                retry_policy, auth_type = config.retry_policy, config.auth_type
+                auth_type, retry_policy, version = config.auth_type, config.retry_policy, config.version
                 if not credentials:
                     credentials = config.credentials
             else:
-                retry_policy, auth_type = None, None
-            self.ad_response, self.protocol = discover(
-                email=primary_smtp_address, credentials=credentials, auth_type=auth_type, retry_policy=retry_policy
-            )
+                auth_type, retry_policy, version = None, None, None
+            self.ad_response, self.protocol = Autodiscovery(
+                email=primary_smtp_address, credentials=credentials
+            ).discover()
+            # Let's not use the auth_package hint from the AD response. It could be incorrect and we can just guess.
+            self.protocol.config.auth_type = auth_type
+            if retry_policy:
+                self.protocol.config.retry_policy = retry_policy
+            if version:
+                self.protocol.config.version = version
             primary_smtp_address = self.ad_response.autodiscover_smtp_address
         else:
             if not config:
-                raise AttributeError('non-autodiscover requires a config')
+                raise AttributeError("non-autodiscover requires a config")
             self.ad_response = None
             self.protocol = Protocol(config=config)
 
         # Other ways of identifying the account can be added later
         self.identity = Identity(primary_smtp_address=primary_smtp_address)
 
+        # For maintaining affinity in e.g. subscriptions
+        self.affinity_cookie = None
+
         # We may need to override the default server version on a per-account basis because Microsoft may report one
-        # server version up-front but delegate account requests to an older backend server.
-        self.version = self.protocol.version
-        log.debug('Added account: %s', self)
+        # server version up-front but delegate account requests to an older backend server. Create a new instance to
+        # avoid changing the protocol version.
+        self.version = self.protocol.version.copy()
+        log.debug("Added account: %s", self)
 
     @property
     def primary_smtp_address(self):
@@ -339,7 +410,7 @@ class Account:
             # We accept generators, so it's not always convenient for caller to know up-front if 'ids' is empty. Allow
             # empty 'ids' and return early.
             return
-        kwargs['items'] = items
+        kwargs["items"] = items
         yield from service_cls(account=self, chunk_size=chunk_size).call(**kwargs)
 
     def export(self, items, chunk_size=None):
@@ -350,9 +421,7 @@ class Account:
 
         :return: A list of strings, the exported representation of the object
         """
-        return list(
-            self._consume_item_service(service_cls=ExportItems, items=items, chunk_size=chunk_size, kwargs={})
-        )
+        return list(self._consume_item_service(service_cls=ExportItems, items=items, chunk_size=chunk_size, kwargs={}))
 
     def upload(self, data, chunk_size=None):
         """Upload objects retrieved from an export to the given folders.
@@ -374,12 +443,11 @@ class Account:
           -> [("idA", "changekey"), ("idB", "changekey"), ("idC", "changekey")]
         """
         items = ((f, (None, False, d) if isinstance(d, str) else d) for f, d in data)
-        return list(
-            self._consume_item_service(service_cls=UploadItems, items=items, chunk_size=chunk_size, kwargs={})
-        )
+        return list(self._consume_item_service(service_cls=UploadItems, items=items, chunk_size=chunk_size, kwargs={}))
 
-    def bulk_create(self, folder, items, message_disposition=SAVE_ONLY, send_meeting_invitations=SEND_TO_NONE,
-                    chunk_size=None):
+    def bulk_create(
+        self, folder, items, message_disposition=SAVE_ONLY, send_meeting_invitations=SEND_TO_NONE, chunk_size=None
+    ):
         """Create new items in 'folder'.
 
         :param folder: the folder to create the items in
@@ -396,23 +464,36 @@ class Account:
         """
         if isinstance(items, QuerySet):
             # bulk_create() on a queryset does not make sense because it returns items that have already been created
-            raise ValueError('Cannot bulk create items from a QuerySet')
+            raise ValueError("Cannot bulk create items from a QuerySet")
         log.debug(
-            'Adding items for %s (folder %s, message_disposition: %s, send_meeting_invitations: %s)',
+            "Adding items for %s (folder %s, message_disposition: %s, send_meeting_invitations: %s)",
             self,
             folder,
             message_disposition,
             send_meeting_invitations,
         )
-        return list(self._consume_item_service(service_cls=CreateItem, items=items, chunk_size=chunk_size, kwargs=dict(
-            folder=folder,
-            message_disposition=message_disposition,
-            send_meeting_invitations=send_meeting_invitations,
-        )))
+        return list(
+            self._consume_item_service(
+                service_cls=CreateItem,
+                items=items,
+                chunk_size=chunk_size,
+                kwargs=dict(
+                    folder=folder,
+                    message_disposition=message_disposition,
+                    send_meeting_invitations=send_meeting_invitations,
+                ),
+            )
+        )
 
-    def bulk_update(self, items, conflict_resolution=AUTO_RESOLVE, message_disposition=SAVE_ONLY,
-                    send_meeting_invitations_or_cancellations=SEND_TO_NONE, suppress_read_receipts=True,
-                    chunk_size=None):
+    def bulk_update(
+        self,
+        items,
+        conflict_resolution=AUTO_RESOLVE,
+        message_disposition=SAVE_ONLY,
+        send_meeting_invitations_or_cancellations=SEND_TO_NONE,
+        suppress_read_receipts=True,
+        chunk_size=None,
+    ):
         """Bulk update existing items.
 
         :param items: a list of (Item, fieldnames) tuples, where 'Item' is an Item object, and 'fieldnames' is a list
@@ -432,23 +513,37 @@ class Account:
         # fact, it could be dangerous if the queryset contains an '.only()'. This would wipe out certain fields
         # entirely.
         if isinstance(items, QuerySet):
-            raise ValueError('Cannot bulk update on a queryset')
+            raise ValueError("Cannot bulk update on a queryset")
         log.debug(
-            'Updating items for %s (conflict_resolution %s, message_disposition: %s, send_meeting_invitations: %s)',
+            "Updating items for %s (conflict_resolution %s, message_disposition: %s, send_meeting_invitations: %s)",
             self,
             conflict_resolution,
             message_disposition,
             send_meeting_invitations_or_cancellations,
         )
-        return list(self._consume_item_service(service_cls=UpdateItem, items=items, chunk_size=chunk_size, kwargs=dict(
-            conflict_resolution=conflict_resolution,
-            message_disposition=message_disposition,
-            send_meeting_invitations_or_cancellations=send_meeting_invitations_or_cancellations,
-            suppress_read_receipts=suppress_read_receipts,
-        )))
+        return list(
+            self._consume_item_service(
+                service_cls=UpdateItem,
+                items=items,
+                chunk_size=chunk_size,
+                kwargs=dict(
+                    conflict_resolution=conflict_resolution,
+                    message_disposition=message_disposition,
+                    send_meeting_invitations_or_cancellations=send_meeting_invitations_or_cancellations,
+                    suppress_read_receipts=suppress_read_receipts,
+                ),
+            )
+        )
 
-    def bulk_delete(self, ids, delete_type=HARD_DELETE, send_meeting_cancellations=SEND_TO_NONE,
-                    affected_task_occurrences=ALL_OCCURRENCIES, suppress_read_receipts=True, chunk_size=None):
+    def bulk_delete(
+        self,
+        ids,
+        delete_type=HARD_DELETE,
+        send_meeting_cancellations=SEND_TO_NONE,
+        affected_task_occurrences=ALL_OCCURRENCES,
+        suppress_read_receipts=True,
+        chunk_size=None,
+    ):
         """Bulk delete items.
 
         :param ids: an iterable of either (id, changekey) tuples or Item objects.
@@ -457,26 +552,31 @@ class Account:
         :param send_meeting_cancellations: only applicable to CalendarItem. Possible values are specified in
             SEND_MEETING_CANCELLATIONS_CHOICES. (Default value = SEND_TO_NONE)
         :param affected_task_occurrences: only applicable for recurring Task items. Possible values are specified in
-            AFFECTED_TASK_OCCURRENCES_CHOICES. (Default value = ALL_OCCURRENCIES)
+            AFFECTED_TASK_OCCURRENCES_CHOICES. (Default value = ALL_OCCURRENCES)
         :param suppress_read_receipts: only supported from Exchange 2013. True or False. (Default value = True)
         :param chunk_size: The number of items to send to the server in a single request (Default value = None)
 
         :return: a list of either True or exception instances, in the same order as the input
         """
         log.debug(
-            'Deleting items for %s (delete_type: %s, send_meeting_invitations: %s, affected_task_occurences: %s)',
+            "Deleting items for %s (delete_type: %s, send_meeting_invitations: %s, affected_task_occurrences: %s)",
             self,
             delete_type,
             send_meeting_cancellations,
             affected_task_occurrences,
         )
         return list(
-            self._consume_item_service(service_cls=DeleteItem, items=ids, chunk_size=chunk_size, kwargs=dict(
-                delete_type=delete_type,
-                send_meeting_cancellations=send_meeting_cancellations,
-                affected_task_occurrences=affected_task_occurrences,
-                suppress_read_receipts=suppress_read_receipts,
-            ))
+            self._consume_item_service(
+                service_cls=DeleteItem,
+                items=ids,
+                chunk_size=chunk_size,
+                kwargs=dict(
+                    delete_type=delete_type,
+                    send_meeting_cancellations=send_meeting_cancellations,
+                    affected_task_occurrences=affected_task_occurrences,
+                    suppress_read_receipts=suppress_read_receipts,
+                ),
+            )
         )
 
     def bulk_send(self, ids, save_copy=True, copy_to_folder=None, chunk_size=None):
@@ -494,9 +594,14 @@ class Account:
         if save_copy and not copy_to_folder:
             copy_to_folder = self.sent  # 'Sent' is default EWS behaviour
         return list(
-            self._consume_item_service(service_cls=SendItem, items=ids, chunk_size=chunk_size, kwargs=dict(
-                saved_item_folder=copy_to_folder,
-            ))
+            self._consume_item_service(
+                service_cls=SendItem,
+                items=ids,
+                chunk_size=chunk_size,
+                kwargs=dict(
+                    saved_item_folder=copy_to_folder,
+                ),
+            )
         )
 
     def bulk_copy(self, ids, to_folder, chunk_size=None):
@@ -508,9 +613,16 @@ class Account:
 
         :return: Status for each send operation, in the same order as the input
         """
-        return list(self._consume_item_service(service_cls=CopyItem, items=ids, chunk_size=chunk_size, kwargs=dict(
-            to_folder=to_folder,
-        )))
+        return list(
+            self._consume_item_service(
+                service_cls=CopyItem,
+                items=ids,
+                chunk_size=chunk_size,
+                kwargs=dict(
+                    to_folder=to_folder,
+                ),
+            )
+        )
 
     def bulk_move(self, ids, to_folder, chunk_size=None):
         """Move items to another folder.
@@ -522,9 +634,16 @@ class Account:
         :return: The new IDs of the moved items, in the same order as the input. If 'to_folder' is a public folder or a
           folder in a different mailbox, an empty list is returned.
         """
-        return list(self._consume_item_service(service_cls=MoveItem, items=ids, chunk_size=chunk_size, kwargs=dict(
-            to_folder=to_folder,
-        )))
+        return list(
+            self._consume_item_service(
+                service_cls=MoveItem,
+                items=ids,
+                chunk_size=chunk_size,
+                kwargs=dict(
+                    to_folder=to_folder,
+                ),
+            )
+        )
 
     def bulk_archive(self, ids, to_folder, chunk_size=None):
         """Archive items to a folder in the archive mailbox. An archive mailbox must be enabled in order for this
@@ -536,9 +655,15 @@ class Account:
 
         :return: A list containing True or an exception instance in stable order of the requested items
         """
-        return list(self._consume_item_service(service_cls=ArchiveItem, items=ids, chunk_size=chunk_size, kwargs=dict(
-                to_folder=to_folder,
-            ))
+        return list(
+            self._consume_item_service(
+                service_cls=ArchiveItem,
+                items=ids,
+                chunk_size=chunk_size,
+                kwargs=dict(
+                    to_folder=to_folder,
+                ),
+            )
         )
 
     def bulk_mark_as_junk(self, ids, is_junk, move_item, chunk_size=None):
@@ -552,10 +677,17 @@ class Account:
         :return: A list containing the new IDs of the moved items, if items were moved, or True, or an exception
           instance, in stable order of the requested items.
         """
-        return list(self._consume_item_service(service_cls=MarkAsJunk, items=ids, chunk_size=chunk_size, kwargs=dict(
-            is_junk=is_junk,
-            move_item=move_item,
-        )))
+        return list(
+            self._consume_item_service(
+                service_cls=MarkAsJunk,
+                items=ids,
+                chunk_size=chunk_size,
+                kwargs=dict(
+                    is_junk=is_junk,
+                    move_item=move_item,
+                ),
+            )
+        )
 
     def fetch(self, ids, folder=None, only_fields=None, chunk_size=None):
         """Fetch items by ID.
@@ -580,13 +712,19 @@ class Account:
             for field in only_fields:
                 validation_folder.validate_item_field(field=field, version=self.version)
             # Remove ItemId and ChangeKey. We get them unconditionally
-            additional_fields = {f for f in validation_folder.normalize_fields(fields=only_fields)
-                                 if not f.field.is_attribute}
+            additional_fields = {
+                f for f in validation_folder.normalize_fields(fields=only_fields) if not f.field.is_attribute
+            }
         # Always use IdOnly here, because AllProperties doesn't actually get *all* properties
-        yield from self._consume_item_service(service_cls=GetItem, items=ids, chunk_size=chunk_size, kwargs=dict(
+        yield from self._consume_item_service(
+            service_cls=GetItem,
+            items=ids,
+            chunk_size=chunk_size,
+            kwargs=dict(
                 additional_fields=additional_fields,
                 shape=ID_ONLY,
-        ))
+            ),
+        )
 
     def fetch_personas(self, ids):
         """Fetch personas by ID.
@@ -602,33 +740,23 @@ class Account:
             # We accept generators, so it's not always convenient for caller to know up-front if 'ids' is empty. Allow
             # empty 'ids' and return early.
             return
-        # GetPersona only accepts one persona ID per request. Crazy.
-        svc = GetPersona(account=self)
-        for i in ids:
-            yield svc.call(persona=i)
+        yield from GetPersona(account=self).call(personas=ids)
 
     @property
     def mail_tips(self):
         """See self.oof_settings about caching considerations."""
-        # mail_tips_requested must be one of properties.MAIL_TIPS_TYPES
         return GetMailTips(protocol=self.protocol).get(
             sending_as=SendingAs(email_address=self.primary_smtp_address),
             recipients=[Mailbox(email_address=self.primary_smtp_address)],
-            mail_tips_requested='All',
+            mail_tips_requested="All",
         )
 
     @property
     def delegates(self):
         """Return a list of DelegateUser objects representing the delegates that are set on this account."""
-        delegates = []
-        for d in GetDelegate(account=self).call(user_ids=None, include_permissions=True):
-            if isinstance(d, Exception):
-                raise d
-            delegates.append(d)
-        return delegates
+        return list(GetDelegate(account=self).call(user_ids=None, include_permissions=True))
 
     def __str__(self):
-        txt = '%s' % self.primary_smtp_address
         if self.fullname:
-            txt += ' (%s)' % self.fullname
-        return txt
+            return f"{self.primary_smtp_address} ({self.fullname})"
+        return self.primary_smtp_address
