@@ -1,15 +1,14 @@
+import abc
 import logging
 
 from cached_property import threaded_cached_property
 
-from .queryset import FOLDER_TRAVERSAL_CHOICES
+from ..errors import InvalidTypeError
 from ..fields import FieldPath, InvalidField
-from ..items import Persona, ITEM_TRAVERSAL_CHOICES, SHAPE_CHOICES, ID_ONLY
+from ..items import ID_ONLY, Persona
 from ..properties import CalendarView
-from ..queryset import QuerySet, SearchableMixIn, Q
+from ..queryset import Q, QuerySet, SearchableMixIn
 from ..restriction import Restriction
-from ..services import FindFolder, GetFolder, FindItem, FindPeople, SyncFolderItems, SyncFolderHierarchy, \
-    SubscribeToPull, SubscribeToPush, SubscribeToStreaming
 from ..util import require_account
 
 log = logging.getLogger(__name__)
@@ -27,7 +26,7 @@ class FolderCollection(SearchableMixIn):
     """A class that implements an API for searching folders."""
 
     # These fields are required in a FindFolder or GetFolder call to properly identify folder types
-    REQUIRED_FOLDER_FIELDS = ('name', 'folder_class')
+    REQUIRED_FOLDER_FIELDS = ("name", "folder_class")
 
     def __init__(self, account, folders):
         """Implement a search API on a collection of folders.
@@ -41,7 +40,7 @@ class FolderCollection(SearchableMixIn):
     @threaded_cached_property
     def folders(self):
         # Resolve the list of folders, in case it's a generator
-        return list(self._folders)
+        return tuple(self._folders)
 
     def __len__(self):
         return len(self.folders)
@@ -126,7 +125,8 @@ class FolderCollection(SearchableMixIn):
         return tuple(item_model for folder in self.folders for item_model in folder.supported_item_models)
 
     def validate_item_field(self, field, version):
-        # For each field, check if the field is valid for any of the item models supported by this folder
+        # Takes a fieldname, Field or FieldPath object pointing to an item field, and checks that it is valid
+        # for the item types supported by this folder collection.
         for item_model in self.supported_item_models:
             try:
                 item_model.validate_field(field=field, version=version)
@@ -134,14 +134,45 @@ class FolderCollection(SearchableMixIn):
             except InvalidField:
                 continue
         else:
-            raise InvalidField("%r is not a valid field on %s" % (field, self.supported_item_models))
+            raise InvalidField(f"{field!r} is not a valid field on {self.supported_item_models}")
 
-    def find_items(self, q, shape=ID_ONLY, depth=None, additional_fields=None, order_fields=None,
-                   calendar_view=None, page_size=None, max_items=None, offset=0):
+    def _rinse_args(self, q, depth, additional_fields, field_validator):
+        if depth is None:
+            depth = self._get_default_item_traversal_depth()
+        if additional_fields:
+            for f in additional_fields:
+                field_validator(field=f, version=self.account.version)
+                if f.field.is_complex:
+                    raise ValueError(f"Field {f.field.name!r} not supported for this service")
+
+        # Build up any restrictions
+        if q.is_empty():
+            restriction = None
+            query_string = None
+        elif q.query_string:
+            restriction = None
+            query_string = Restriction(q, folders=self.folders, applies_to=Restriction.ITEMS)
+        else:
+            restriction = Restriction(q, folders=self.folders, applies_to=Restriction.ITEMS)
+            query_string = None
+        return depth, restriction, query_string
+
+    def find_items(
+        self,
+        q,
+        shape=ID_ONLY,
+        depth=None,
+        additional_fields=None,
+        order_fields=None,
+        calendar_view=None,
+        page_size=None,
+        max_items=None,
+        offset=0,
+    ):
         """Private method to call the FindItem service.
 
         :param q: a Q instance containing any restrictions
-        :param shape: controls whether to return (id, chanegkey) tuples or Item objects. If additional_fields is
+        :param shape: controls whether to return (id, changekey) tuples or Item objects. If additional_fields is
           non-null, we always return Item objects. (Default value = ID_ONLY)
         :param depth: controls the whether to return soft-deleted items or not. (Default value = None)
         :param additional_fields: the extra properties we want on the return objects. Default is no properties. Be aware
@@ -154,38 +185,22 @@ class FolderCollection(SearchableMixIn):
 
         :return: a generator for the returned item IDs or items
         """
+        from ..services import FindItem
+
         if not self.folders:
-            log.debug('Folder list is empty')
+            log.debug("Folder list is empty")
             return
         if q.is_never():
-            log.debug('Query will never return results')
+            log.debug("Query will never return results")
             return
-        if shape not in SHAPE_CHOICES:
-            raise ValueError("'shape' %s must be one of %s" % (shape, SHAPE_CHOICES))
-        if depth is None:
-            depth = self._get_default_item_traversal_depth()
-        if depth not in ITEM_TRAVERSAL_CHOICES:
-            raise ValueError("'depth' %s must be one of %s" % (depth, ITEM_TRAVERSAL_CHOICES))
-        if additional_fields:
-            for f in additional_fields:
-                self.validate_item_field(field=f, version=self.account.version)
-                if f.field.is_complex:
-                    raise ValueError("find_items() does not support field '%s'. Use fetch() instead" % f.field.name)
+        depth, restriction, query_string = self._rinse_args(
+            q=q, depth=depth, additional_fields=additional_fields, field_validator=self.validate_item_field
+        )
         if calendar_view is not None and not isinstance(calendar_view, CalendarView):
-            raise ValueError("'calendar_view' %s must be a CalendarView instance" % calendar_view)
+            raise InvalidTypeError("calendar_view", calendar_view, CalendarView)
 
-        # Build up any restrictions
-        if q.is_empty():
-            restriction = None
-            query_string = None
-        elif q.query_string:
-            restriction = None
-            query_string = Restriction(q, folders=self.folders, applies_to=Restriction.ITEMS)
-        else:
-            restriction = Restriction(q, folders=self.folders, applies_to=Restriction.ITEMS)
-            query_string = None
         log.debug(
-            'Finding %s items in folders %s (shape: %s, depth: %s, additional_fields: %s, restriction: %s)',
+            "Finding %s items in folders %s (shape: %s, depth: %s, additional_fields: %s, restriction: %s)",
             self.account,
             self.folders,
             shape,
@@ -193,7 +208,7 @@ class FolderCollection(SearchableMixIn):
             additional_fields,
             restriction.q if restriction else None,
         )
-        yield from FindItem(account=self.account, chunk_size=page_size).call(
+        yield from FindItem(account=self.account, page_size=page_size).call(
             folders=self.folders,
             additional_fields=additional_fields,
             restriction=restriction,
@@ -208,18 +223,27 @@ class FolderCollection(SearchableMixIn):
 
     def _get_single_folder(self):
         if len(self.folders) > 1:
-            raise ValueError('Syncing folder hierarchy can only be done on a single folder')
+            raise ValueError("Syncing folder hierarchy can only be done on a single folder")
         if not self.folders:
-            log.debug('Folder list is empty')
+            log.debug("Folder list is empty")
             return None
         return self.folders[0]
 
-    def find_people(self, q, shape=ID_ONLY, depth=None, additional_fields=None, order_fields=None,
-                    page_size=None, max_items=None, offset=0):
+    def find_people(
+        self,
+        q,
+        shape=ID_ONLY,
+        depth=None,
+        additional_fields=None,
+        order_fields=None,
+        page_size=None,
+        max_items=None,
+        offset=0,
+    ):
         """Private method to call the FindPeople service.
 
         :param q: a Q instance containing any restrictions
-        :param shape: controls whether to return (id, chanegkey) tuples or Persona objects. If additional_fields is
+        :param shape: controls whether to return (id, changekey) tuples or Persona objects. If additional_fields is
           non-null, we always return Persona objects. (Default value = ID_ONLY)
         :param depth: controls the whether to return soft-deleted items or not. (Default value = None)
         :param additional_fields: the extra properties we want on the return objects. Default is no properties.
@@ -230,49 +254,32 @@ class FolderCollection(SearchableMixIn):
 
         :return: a generator for the returned personas
         """
-        folder = self._get_single_folder()
-        if not folder:
-            return
-        if q.is_never():
-            log.debug('Query will never return results')
-            return
-        if shape not in SHAPE_CHOICES:
-            raise ValueError("'shape' %s must be one of %s" % (shape, SHAPE_CHOICES))
-        if depth is None:
-            depth = self._get_default_item_traversal_depth()
-        if depth not in ITEM_TRAVERSAL_CHOICES:
-            raise ValueError("'depth' %s must be one of %s" % (depth, ITEM_TRAVERSAL_CHOICES))
-        if additional_fields:
-            for f in additional_fields:
-                Persona.validate_field(field=f, version=self.account.version)
-                if f.field.is_complex:
-                    raise ValueError("find_people() does not support field '%s'" % f.field.name)
+        from ..services import FindPeople
 
-        # Build up any restrictions
-        if q.is_empty():
-            restriction = None
-            query_string = None
-        elif q.query_string:
-            restriction = None
-            query_string = Restriction(q, folders=[folder], applies_to=Restriction.ITEMS)
-        else:
-            restriction = Restriction(q, folders=[folder], applies_to=Restriction.ITEMS)
-            query_string = None
-        yield from FindPeople(account=self.account, chunk_size=page_size).call(
-                folder=folder,
-                additional_fields=additional_fields,
-                restriction=restriction,
-                order_fields=order_fields,
-                shape=shape,
-                query_string=query_string,
-                depth=depth,
-                max_items=max_items,
-                offset=offset,
+        folder = self._get_single_folder()
+        if q.is_never():
+            log.debug("Query will never return results")
+            return
+        depth, restriction, query_string = self._rinse_args(
+            q=q, depth=depth, additional_fields=additional_fields, field_validator=Persona.validate_field
+        )
+
+        yield from FindPeople(account=self.account, page_size=page_size).call(
+            folder=folder,
+            additional_fields=additional_fields,
+            restriction=restriction,
+            order_fields=order_fields,
+            shape=shape,
+            query_string=query_string,
+            depth=depth,
+            max_items=max_items,
+            offset=offset,
         )
 
     def get_folder_fields(self, target_cls, is_complex=None):
         return {
-            FieldPath(field=f) for f in target_cls.supported_fields(version=self.account.version)
+            FieldPath(field=f)
+            for f in target_cls.supported_fields(version=self.account.version)
             if is_complex is None or f.is_complex is is_complex
         }
 
@@ -281,16 +288,17 @@ class FolderCollection(SearchableMixIn):
         # both folder types in self.folders, raise an error so we don't risk losing some fields in the query.
         from .base import Folder
         from .roots import RootOfHierarchy
+
         has_roots = False
         has_non_roots = False
         for f in self.folders:
             if isinstance(f, RootOfHierarchy):
                 if has_non_roots:
-                    raise ValueError('Cannot call GetFolder on a mix of folder types: {}'.format(self.folders))
+                    raise ValueError(f"Cannot call GetFolder on a mix of folder types: {self.folders}")
                 has_roots = True
             else:
                 if has_roots:
-                    raise ValueError('Cannot call GetFolder on a mix of folder types: {}'.format(self.folders))
+                    raise ValueError(f"Cannot call GetFolder on a mix of folder types: {self.folders}")
                 has_non_roots = True
         return RootOfHierarchy if has_roots else Folder
 
@@ -299,85 +307,89 @@ class FolderCollection(SearchableMixIn):
         if len(unique_depths) == 1:
             return unique_depths.pop()
         raise ValueError(
-            'Folders in this collection do not have a common %s value. You need to define an explicit traversal depth'
-            'with QuerySet.depth() (values: %s)' % (traversal_attr, unique_depths)
+            f"Folders in this collection do not have a common {traversal_attr} value. You need to define an explicit "
+            f"traversal depth with QuerySet.depth() (values: {unique_depths})"
         )
 
     def _get_default_item_traversal_depth(self):
         # When searching folders, some folders require 'Shallow' and others 'Associated' traversal depth.
-        return self._get_default_traversal_depth('DEFAULT_ITEM_TRAVERSAL_DEPTH')
+        return self._get_default_traversal_depth("DEFAULT_ITEM_TRAVERSAL_DEPTH")
 
     def _get_default_folder_traversal_depth(self):
         # When searching folders, some folders require 'Shallow' and others 'Deep' traversal depth.
-        return self._get_default_traversal_depth('DEFAULT_FOLDER_TRAVERSAL_DEPTH')
+        return self._get_default_traversal_depth("DEFAULT_FOLDER_TRAVERSAL_DEPTH")
 
     def resolve(self):
         # Looks up the folders or folder IDs in the collection and returns full Folder instances with all fields set.
         from .base import BaseFolder
+
         resolveable_folders = []
         for f in self.folders:
             if isinstance(f, BaseFolder) and not f.get_folder_allowed:
-                log.debug('GetFolder not allowed on folder %s. Non-complex fields must be fetched with FindFolder', f)
+                log.debug("GetFolder not allowed on folder %s. Non-complex fields must be fetched with FindFolder", f)
                 yield f
             else:
                 resolveable_folders.append(f)
         # Fetch all properties for the remaining folders of folder IDs
         additional_fields = self.get_folder_fields(target_cls=self._get_target_cls(), is_complex=None)
         yield from self.__class__(account=self.account, folders=resolveable_folders).get_folders(
-                additional_fields=additional_fields
+            additional_fields=additional_fields
         )
 
     @require_account
-    def find_folders(self, q=None, shape=ID_ONLY, depth=None, additional_fields=None, page_size=None, max_items=None,
-                     offset=0):
+    def find_folders(
+        self, q=None, shape=ID_ONLY, depth=None, additional_fields=None, page_size=None, max_items=None, offset=0
+    ):
+        from ..services import FindFolder
+
         # 'depth' controls whether to return direct children or recurse into sub-folders
         from .base import BaseFolder, Folder
+
         if q is None:
             q = Q()
         if not self.folders:
-            log.debug('Folder list is empty')
+            log.debug("Folder list is empty")
             return
         if q.is_never():
-            log.debug('Query will never return results')
+            log.debug("Query will never return results")
             return
         if q.is_empty():
             restriction = None
         else:
             restriction = Restriction(q, folders=self.folders, applies_to=Restriction.FOLDERS)
-        if shape not in SHAPE_CHOICES:
-            raise ValueError("'shape' %s must be one of %s" % (shape, SHAPE_CHOICES))
         if depth is None:
             depth = self._get_default_folder_traversal_depth()
-        if depth not in FOLDER_TRAVERSAL_CHOICES:
-            raise ValueError("'depth' %s must be one of %s" % (depth, FOLDER_TRAVERSAL_CHOICES))
         if additional_fields is None:
             # Default to all non-complex properties. Subfolders will always be of class Folder
             additional_fields = self.get_folder_fields(target_cls=Folder, is_complex=False)
         else:
             for f in additional_fields:
                 if f.field.is_complex:
-                    raise ValueError("find_folders() does not support field '%s'. Use get_folders()." % f.field.name)
+                    raise ValueError(f"find_folders() does not support field {f.field.name!r}. Use get_folders().")
 
         # Add required fields
         additional_fields.update(
             (FieldPath(field=BaseFolder.get_field_by_fieldname(f)) for f in self.REQUIRED_FOLDER_FIELDS)
         )
 
-        yield from FindFolder(account=self.account, chunk_size=page_size).call(
-                folders=self.folders,
-                additional_fields=additional_fields,
-                restriction=restriction,
-                shape=shape,
-                depth=depth,
-                max_items=max_items,
-                offset=offset,
+        yield from FindFolder(account=self.account, page_size=page_size).call(
+            folders=self.folders,
+            additional_fields=additional_fields,
+            restriction=restriction,
+            shape=shape,
+            depth=depth,
+            max_items=max_items,
+            offset=offset,
         )
 
     def get_folders(self, additional_fields=None):
+        from ..services import GetFolder
+
         # Expand folders with their full set of properties
         from .base import BaseFolder
+
         if not self.folders:
-            log.debug('Folder list is empty')
+            log.debug("Folder list is empty")
             return
         if additional_fields is None:
             # Default to all complex properties
@@ -389,39 +401,78 @@ class FolderCollection(SearchableMixIn):
         )
 
         yield from GetFolder(account=self.account).call(
-                folders=self.folders,
-                additional_fields=additional_fields,
-                shape=ID_ONLY,
+            folders=self.folders,
+            additional_fields=additional_fields,
+            shape=ID_ONLY,
         )
 
-    def subscribe_to_pull(self, event_types=SubscribeToPull.EVENT_TYPES, watermark=None, timeout=60):
+    def subscribe_to_pull(self, event_types=None, watermark=None, timeout=60):
+        from ..services import SubscribeToPull
+
         if not self.folders:
-            log.debug('Folder list is empty')
-            return
-        yield from SubscribeToPull(account=self.account).call(
-            folders=self.folders, event_types=event_types, watermark=watermark, timeout=timeout,
+            log.debug("Folder list is empty")
+            return None
+        if event_types is None:
+            event_types = SubscribeToPull.EVENT_TYPES
+        return SubscribeToPull(account=self.account).get(
+            folders=self.folders,
+            event_types=event_types,
+            watermark=watermark,
+            timeout=timeout,
         )
 
-    def subscribe_to_push(self, callback_url, event_types=SubscribeToPush.EVENT_TYPES, watermark=None,
-                          status_frequency=1):
+    def subscribe_to_push(self, callback_url, event_types=None, watermark=None, status_frequency=1):
+        from ..services import SubscribeToPush
+
         if not self.folders:
-            log.debug('Folder list is empty')
-            return
-        yield from SubscribeToPush(account=self.account).call(
-            folders=self.folders, event_types=event_types, watermark=watermark, status_frequency=status_frequency,
+            log.debug("Folder list is empty")
+            return None
+        if event_types is None:
+            event_types = SubscribeToPush.EVENT_TYPES
+        return SubscribeToPush(account=self.account).get(
+            folders=self.folders,
+            event_types=event_types,
+            watermark=watermark,
+            status_frequency=status_frequency,
             url=callback_url,
         )
 
-    def subscribe_to_streaming(self, event_types=SubscribeToPush.EVENT_TYPES):
+    def subscribe_to_streaming(self, event_types=None):
+        from ..services import SubscribeToStreaming
+
         if not self.folders:
-            log.debug('Folder list is empty')
-            return
-        yield from SubscribeToStreaming(account=self.account).call(folders=self.folders, event_types=event_types)
+            log.debug("Folder list is empty")
+            return None
+        if event_types is None:
+            event_types = SubscribeToStreaming.EVENT_TYPES
+        return SubscribeToStreaming(account=self.account).get(folders=self.folders, event_types=event_types)
+
+    def pull_subscription(self, **kwargs):
+        return PullSubscription(folder=self, **kwargs)
+
+    def push_subscription(self, **kwargs):
+        return PushSubscription(folder=self, **kwargs)
+
+    def streaming_subscription(self, **kwargs):
+        return StreamingSubscription(folder=self, **kwargs)
+
+    def unsubscribe(self, subscription_id):
+        """Unsubscribe. Only applies to pull and streaming notifications.
+
+        :param subscription_id: A subscription ID as acquired by .subscribe_to_[pull|streaming]()
+        :return: True
+
+        This method doesn't need the current collection instance, but it makes sense to keep the method along the other
+        sync methods.
+        """
+        from ..services import Unsubscribe
+
+        return Unsubscribe(account=self.account).get(subscription_id=subscription_id)
 
     def sync_items(self, sync_state=None, only_fields=None, ignore=None, max_changes_returned=None, sync_scope=None):
+        from ..services import SyncFolderItems
+
         folder = self._get_single_folder()
-        if not folder:
-            return
         if only_fields is None:
             # We didn't restrict list of field paths. Get all fields from the server, including extended properties.
             additional_fields = {FieldPath(field=f) for f in folder.allowed_item_fields(version=self.account.version)}
@@ -451,17 +502,20 @@ class FolderCollection(SearchableMixIn):
         raise SyncCompleted(sync_state=svc.sync_state)
 
     def sync_hierarchy(self, sync_state=None, only_fields=None):
+        from ..services import SyncFolderHierarchy
+
         folder = self._get_single_folder()
-        if not folder:
-            return
         if only_fields is None:
             # We didn't restrict list of field paths. Get all fields from the server, including extended properties.
             additional_fields = {FieldPath(field=f) for f in folder.supported_fields(version=self.account.version)}
         else:
-            for f in only_fields:
-                folder.validate_field(field=f, version=self.account.version)
-            # Remove ItemId and ChangeKey. We get them unconditionally
-            additional_fields = {f for f in folder.normalize_fields(fields=only_fields) if not f.field.is_attribute}
+            additional_fields = set()
+            for field_name in only_fields:
+                folder.validate_field(field=field_name, version=self.account.version)
+                f = folder.get_field_by_fieldname(fieldname=field_name)
+                if not f.is_attribute:
+                    # Remove ItemId and ChangeKey. We get them unconditionally
+                    additional_fields.add(FieldPath(field=f))
 
         # Add required fields
         additional_fields.update(
@@ -483,3 +537,40 @@ class FolderCollection(SearchableMixIn):
             if svc.includes_last_item_in_range:  # Try again if there are more items
                 break
         raise SyncCompleted(sync_state=svc.sync_state)
+
+
+class BaseSubscription(metaclass=abc.ABCMeta):
+    def __init__(self, folder, **subscription_kwargs):
+        self.folder = folder
+        self.subscription_kwargs = subscription_kwargs
+        self.subscription_id = None
+
+    @abc.abstractmethod
+    def __enter__(self):
+        """Create the subscription"""
+
+    def __exit__(self, *args, **kwargs):
+        self.folder.unsubscribe(subscription_id=self.subscription_id)
+        self.subscription_id = None
+
+
+class PullSubscription(BaseSubscription):
+    def __enter__(self):
+        self.subscription_id, watermark = self.folder.subscribe_to_pull(**self.subscription_kwargs)
+        return self.subscription_id, watermark
+
+
+class PushSubscription(BaseSubscription):
+    def __enter__(self):
+        self.subscription_id, watermark = self.folder.subscribe_to_push(**self.subscription_kwargs)
+        return self.subscription_id, watermark
+
+    def __exit__(self, *args, **kwargs):
+        # Cannot unsubscribe to push subscriptions
+        pass
+
+
+class StreamingSubscription(BaseSubscription):
+    def __enter__(self):
+        self.subscription_id = self.folder.subscribe_to_streaming(**self.subscription_kwargs)
+        return self.subscription_id
