@@ -1,72 +1,73 @@
 import abc
 import logging
-import warnings
 from copy import deepcopy
 from itertools import islice
 
-from .errors import MultipleObjectsReturned, DoesNotExist
-from .fields import FieldPath, FieldOrder
-from .items import CalendarItem, ID_ONLY
+from .errors import DoesNotExist, ErrorItemNotFound, InvalidEnumValue, InvalidTypeError, MultipleObjectsReturned
+from .fields import FieldOrder, FieldPath
+from .items import ID_ONLY, CalendarItem
 from .properties import InvalidField
 from .restriction import Q
-from .services import CHUNK_SIZE
 from .version import EXCHANGE_2010
 
 log = logging.getLogger(__name__)
 
+MISSING_ITEM_ERRORS = (ErrorItemNotFound,)
+
 
 class SearchableMixIn:
     """Implement a search API for inheritance."""
 
     @abc.abstractmethod
     def get(self, *args, **kwargs):
-        pass
+        """Return a single object"""
 
     @abc.abstractmethod
     def all(self):
-        pass
+        """Return all objects, unfiltered"""
 
     @abc.abstractmethod
     def none(self):
-        pass
+        """Return an empty result"""
 
     @abc.abstractmethod
     def filter(self, *args, **kwargs):
-        pass
+        """Apply filters to a query"""
 
     @abc.abstractmethod
     def exclude(self, *args, **kwargs):
-        pass
+        """Apply filters to a query"""
 
     @abc.abstractmethod
     def people(self):
-        pass
+        """Search for personas"""
 
 
 class QuerySet(SearchableMixIn):
-    """A Django QuerySet-like class for querying items. Defers queries until the QuerySet is consumed. Supports chaining to
-    build up complex queries.
+    """A Django QuerySet-like class for querying items. Defers queries until the QuerySet is consumed. Supports
+    chaining to build up complex queries.
 
     Django QuerySet documentation: https://docs.djangoproject.com/en/dev/ref/models/querysets/
     """
 
-    VALUES = 'values'
-    VALUES_LIST = 'values_list'
-    FLAT = 'flat'
-    NONE = 'none'
+    VALUES = "values"
+    VALUES_LIST = "values_list"
+    FLAT = "flat"
+    NONE = "none"
     RETURN_TYPES = (VALUES, VALUES_LIST, FLAT, NONE)
 
-    ITEM = 'item'
-    PERSONA = 'persona'
+    ITEM = "item"
+    PERSONA = "persona"
     REQUEST_TYPES = (ITEM, PERSONA)
 
     def __init__(self, folder_collection, request_type=ITEM):
         from .folders import FolderCollection
+
         if not isinstance(folder_collection, FolderCollection):
-            raise ValueError("folder_collection value '%s' must be a FolderCollection instance" % folder_collection)
+            raise InvalidTypeError("folder_collection", folder_collection, FolderCollection)
         self.folder_collection = folder_collection  # A FolderCollection instance
         if request_type not in self.REQUEST_TYPES:
-            raise ValueError("'request_type' %r must be one of %s" % (request_type, self.REQUEST_TYPES))
+            raise InvalidEnumValue("request_type", request_type, self.REQUEST_TYPES)
         self.request_type = request_type
         self.q = Q()  # Default to no restrictions
         self.only_fields = None
@@ -74,6 +75,7 @@ class QuerySet(SearchableMixIn):
         self.return_format = self.NONE
         self.calendar_view = None
         self.page_size = None
+        self.chunk_size = None
         self.max_items = None
         self.offset = 0
         self._depth = None
@@ -88,14 +90,6 @@ class QuerySet(SearchableMixIn):
         # items = list(qs)
         # new_qs = qs.exclude(bar='baz')  # This should work, and should fetch from the server
         #
-        if not isinstance(self.q, Q):
-            raise ValueError("self.q value '%s' must be None or a Q instance" % self.q)
-        if not isinstance(self.only_fields, (type(None), tuple)):
-            raise ValueError("self.only_fields value '%s' must be None or a tuple" % self.only_fields)
-        if not isinstance(self.order_fields, (type(None), tuple)):
-            raise ValueError("self.order_fields value '%s' must be None or a tuple" % self.order_fields)
-        if self.return_format not in self.RETURN_TYPES:
-            raise ValueError("self.return_value '%s' must be one of %s" % (self.return_format, self.RETURN_TYPES))
         # Only mutable objects need to be deepcopied. Folder should be the same object
         new_qs = self.__class__(self.folder_collection, request_type=self.request_type)
         new_qs.q = deepcopy(self.q)
@@ -104,6 +98,7 @@ class QuerySet(SearchableMixIn):
         new_qs.return_format = self.return_format
         new_qs.calendar_view = self.calendar_view
         new_qs.page_size = self.page_size
+        new_qs.chunk_size = self.chunk_size
         new_qs.max_items = self.max_items
         new_qs.offset = self.offset
         new_qs._depth = self._depth
@@ -111,6 +106,7 @@ class QuerySet(SearchableMixIn):
 
     def _get_field_path(self, field_path):
         from .items import Persona
+
         if self.request_type == self.PERSONA:
             return FieldPath(field=Persona.get_field_by_fieldname(field_path))
         for folder in self.folder_collection:
@@ -118,41 +114,42 @@ class QuerySet(SearchableMixIn):
                 return FieldPath.from_string(field_path=field_path, folder=folder)
             except InvalidField:
                 pass
-        raise InvalidField("Unknown field path %r on folders %s" % (field_path, self.folder_collection.folders))
+        raise InvalidField(f"Unknown field path {field_path!r} on folders {self.folder_collection.folders}")
 
     def _get_field_order(self, field_path):
         from .items import Persona
+
         if self.request_type == self.PERSONA:
             return FieldOrder(
-                field_path=FieldPath(field=Persona.get_field_by_fieldname(field_path.lstrip('-'))),
-                reverse=field_path.startswith('-'),
+                field_path=FieldPath(field=Persona.get_field_by_fieldname(field_path.lstrip("-"))),
+                reverse=field_path.startswith("-"),
             )
         for folder in self.folder_collection:
             try:
                 return FieldOrder.from_string(field_path=field_path, folder=folder)
             except InvalidField:
                 pass
-        raise InvalidField("Unknown field path %r on folders %s" % (field_path, self.folder_collection.folders))
+        raise InvalidField(f"Unknown field path {field_path!r} on folders {self.folder_collection.folders}")
 
     @property
     def _id_field(self):
-        return self._get_field_path('id')
+        return self._get_field_path("id")
 
     @property
     def _changekey_field(self):
-        return self._get_field_path('changekey')
+        return self._get_field_path("changekey")
 
     def _additional_fields(self):
         if not isinstance(self.only_fields, tuple):
-            raise ValueError("'only_fields' value %r must be a tuple" % self.only_fields)
+            raise InvalidTypeError("only_fields", self.only_fields, tuple)
         # Remove ItemId and ChangeKey. We get them unconditionally
         additional_fields = {f for f in self.only_fields if not f.field.is_attribute}
         if self.request_type != self.ITEM:
             return additional_fields
 
         # For CalendarItem items, we want to inject internal timezone fields into the requested fields.
-        has_start = 'start' in {f.field.name for f in additional_fields}
-        has_end = 'end' in {f.field.name for f in additional_fields}
+        has_start = "start" in {f.field.name for f in additional_fields}
+        has_end = "end" in {f.field.name for f in additional_fields}
         meeting_tz_field, start_tz_field, end_tz_field = CalendarItem.timezone_fields()
         if self.folder_collection.account.version.build < EXCHANGE_2010:
             if has_start or has_end:
@@ -213,31 +210,33 @@ class QuerySet(SearchableMixIn):
         )
         if self.request_type == self.PERSONA:
             if complex_fields_requested:
-                find_kwargs['additional_fields'] = None
+                find_kwargs["additional_fields"] = None
                 items = self.folder_collection.account.fetch_personas(
                     ids=self.folder_collection.find_people(self.q, **find_kwargs)
                 )
             else:
                 if not additional_fields:
-                    find_kwargs['additional_fields'] = None
+                    find_kwargs["additional_fields"] = None
                 items = self.folder_collection.find_people(self.q, **find_kwargs)
         else:
-            find_kwargs['calendar_view'] = self.calendar_view
+            find_kwargs["calendar_view"] = self.calendar_view
             if complex_fields_requested:
                 # The FindItem service does not support complex field types. Tell find_items() to return
                 # (id, changekey) tuples, and pass that to fetch().
-                find_kwargs['additional_fields'] = None
-                items = self.folder_collection.account.fetch(
+                find_kwargs["additional_fields"] = None
+                unfiltered_items = self.folder_collection.account.fetch(
                     ids=self.folder_collection.find_items(self.q, **find_kwargs),
                     only_fields=additional_fields,
-                    chunk_size=self.page_size,
+                    chunk_size=self.chunk_size,
                 )
+                # We may be unlucky that the item disappeared between the FindItem and the GetItem calls
+                items = filter(lambda i: not isinstance(i, MISSING_ITEM_ERRORS), unfiltered_items)
             else:
                 if not additional_fields:
                     # If additional_fields is the empty set, we only requested ID and changekey fields. We can then
                     # take a shortcut by using (shape=ID_ONLY, additional_fields=None) to tell find_items() to return
                     # (id, changekey) tuples. We'll post-process those later.
-                    find_kwargs['additional_fields'] = None
+                    find_kwargs["additional_fields"] = None
                 items = self.folder_collection.find_items(self.q, **find_kwargs)
 
         if not must_sort_clientside:
@@ -250,12 +249,13 @@ class QuerySet(SearchableMixIn):
             try:
                 items = sorted(items, key=lambda i: _get_sort_value_or_default(i, f), reverse=f.reverse)
             except TypeError as e:
-                if 'unorderable types' not in e.args[0]:
+                if "unorderable types" not in e.args[0]:
                     raise
-                raise ValueError((
-                    "Cannot sort on field '%s'. The field has no default value defined, and there are either items "
-                    "with None values for this field, or the query contains exception instances (original error: %s).")
-                                 % (f.field_path, e))
+                raise ValueError(
+                    f"Cannot sort on field {f.field_path!r}. The field has no default value defined, and there are "
+                    f"either items with None values for this field, or the query contains exception instances "
+                    f"(original error: {e})."
+                )
         if not extra_order_fields:
             return items
 
@@ -269,20 +269,18 @@ class QuerySet(SearchableMixIn):
         if self.q.is_never():
             return
 
-        log.debug('Initializing cache')
+        log.debug("Initializing cache")
         yield from self._format_items(items=self._query(), return_format=self.return_format)
 
-    """Do not implement __len__. The implementation of list() tries to preallocate memory by calling __len__ on the
-    given sequence, before calling __iter__. If we implemented __len__, we would end up calling FindItems twice, once
-    to get the result of self.count(), an once to return the actual result.
-
-    Also, according to https://stackoverflow.com/questions/37189968/how-to-have-list-consume-iter-without-calling-len,
-    a __len__ implementation should be cheap. That does not hold for self.count().
-
-    def __len__(self):
-        # This queryset has no cache yet. Call the optimized counting implementation
-        return self.count()
-    """
+    # Do not implement __len__. The implementation of list() tries to preallocate memory by calling __len__ on the
+    # given sequence, before calling __iter__. If we implemented __len__, we would end up calling FindItems twice, once
+    # to get the result of self.count(), and once to return the actual result.
+    #
+    # Also, according to https://stackoverflow.com/questions/37189968/how-to-have-list-consume-iter-without-calling-len,
+    # a __len__ implementation should be cheap. That does not hold for self.count().
+    #
+    # def __len__(self):
+    #     return self.count()
 
     def __getitem__(self, idx_or_slice):
         # Support indexing and slicing. This is non-greedy when possible (slicing start, stop and step are not negative,
@@ -294,7 +292,7 @@ class QuerySet(SearchableMixIn):
     def _getitem_idx(self, idx):
         if idx < 0:
             # Support negative indexes by reversing the queryset and negating the index value
-            reverse_idx = -(idx+1)
+            reverse_idx = -(idx + 1)
             return self.reverse()[reverse_idx]
         # Optimize by setting an exact offset and fetching only 1 item
         new_qs = self._copy_self()
@@ -307,6 +305,8 @@ class QuerySet(SearchableMixIn):
         raise IndexError()
 
     def _getitem_slice(self, s):
+        from .services import PAGE_SIZE
+
         if ((s.start or 0) < 0) or ((s.stop or 0) < 0) or ((s.step or 0) < 0):
             # islice() does not support negative start, stop and step. Make sure cache is full by iterating the full
             # query result, and then slice on the cache.
@@ -320,7 +320,7 @@ class QuerySet(SearchableMixIn):
             new_qs.offset = s.start
         elif s.stop is not None:
             new_qs.max_items = s.stop
-        if new_qs.page_size is None and new_qs.max_items is not None and new_qs.max_items < CHUNK_SIZE:
+        if new_qs.page_size is None and new_qs.max_items is not None and new_qs.max_items < PAGE_SIZE:
             new_qs.page_size = new_qs.max_items
         return islice(new_qs.__iter__(), None, None, s.step)
 
@@ -353,6 +353,7 @@ class QuerySet(SearchableMixIn):
 
     def _as_items(self, iterable):
         from .items import Item
+
         return self._item_yielder(
             iterable=iterable,
             item_func=lambda i: i,
@@ -363,18 +364,18 @@ class QuerySet(SearchableMixIn):
 
     def _as_values(self, iterable):
         if not self.only_fields:
-            raise ValueError('values() requires at least one field name')
+            raise ValueError("values() requires at least one field name")
         return self._item_yielder(
             iterable=iterable,
             item_func=lambda i: {f.path: _get_value_or_default(f, i) for f in self.only_fields},
-            id_only_func=lambda item_id, changekey: {'id': item_id},
-            changekey_only_func=lambda item_id, changekey: {'changekey': changekey},
-            id_and_changekey_func=lambda item_id, changekey: {'id': item_id, 'changekey': changekey},
+            id_only_func=lambda item_id, changekey: {"id": item_id},
+            changekey_only_func=lambda item_id, changekey: {"changekey": changekey},
+            id_and_changekey_func=lambda item_id, changekey: {"id": item_id, "changekey": changekey},
         )
 
     def _as_values_list(self, iterable):
         if not self.only_fields:
-            raise ValueError('values_list() requires at least one field name')
+            raise ValueError("values_list() requires at least one field name")
         return self._item_yielder(
             iterable=iterable,
             item_func=lambda i: tuple(_get_value_or_default(f, i) for f in self.only_fields),
@@ -385,7 +386,7 @@ class QuerySet(SearchableMixIn):
 
     def _as_flat_values_list(self, iterable):
         if not self.only_fields or len(self.only_fields) != 1:
-            raise ValueError('flat=True requires exactly one field name')
+            raise ValueError("flat=True requires exactly one field name")
         return self._item_yielder(
             iterable=iterable,
             item_func=lambda i: _get_value_or_default(self.only_fields[0], i),
@@ -439,7 +440,7 @@ class QuerySet(SearchableMixIn):
         try:
             only_fields = tuple(self._get_field_path(arg) for arg in args)
         except ValueError as e:
-            raise ValueError("%s in only()" % e.args[0])
+            raise ValueError(f"{e.args[0]} in only()")
         new_qs = self._copy_self()
         new_qs.only_fields = only_fields
         return new_qs
@@ -453,7 +454,7 @@ class QuerySet(SearchableMixIn):
         try:
             order_fields = tuple(self._get_field_order(arg) for arg in args)
         except ValueError as e:
-            raise ValueError("%s in order_by()" % e.args[0])
+            raise ValueError(f"{e.args[0]} in order_by()")
         new_qs = self._copy_self()
         new_qs.order_fields = order_fields
         return new_qs
@@ -461,7 +462,7 @@ class QuerySet(SearchableMixIn):
     def reverse(self):
         """Reverses the ordering of the queryset."""
         if not self.order_fields:
-            raise ValueError('Reversing only makes sense if there are order_by fields')
+            raise ValueError("Reversing only makes sense if there are order_by fields")
         new_qs = self._copy_self()
         for f in new_qs.order_fields:
             f.reverse = not f.reverse
@@ -471,7 +472,7 @@ class QuerySet(SearchableMixIn):
         try:
             only_fields = tuple(self._get_field_path(arg) for arg in args)
         except ValueError as e:
-            raise ValueError("%s in values()" % e.args[0])
+            raise ValueError(f"{e.args[0]} in values()")
         new_qs = self._copy_self()
         new_qs.only_fields = only_fields
         new_qs.return_format = self.VALUES
@@ -481,15 +482,15 @@ class QuerySet(SearchableMixIn):
         """Return the values of the specified field names as a list of lists. If called with flat=True and only one
         field name, returns a list of values.
         """
-        flat = kwargs.pop('flat', False)
+        flat = kwargs.pop("flat", False)
         if kwargs:
-            raise AttributeError('Unknown kwargs: %s' % kwargs)
+            raise AttributeError(f"Unknown kwargs: {kwargs}")
         if flat and len(args) != 1:
-            raise ValueError('flat=True requires exactly one field name')
+            raise ValueError("flat=True requires exactly one field name")
         try:
             only_fields = tuple(self._get_field_path(arg) for arg in args)
         except ValueError as e:
-            raise ValueError("%s in values_list()" % e.args[0])
+            raise ValueError(f"{e.args[0]} in values_list()")
         new_qs = self._copy_self()
         new_qs.only_fields = only_fields
         new_qs.return_format = self.FLAT if flat else self.VALUES_LIST
@@ -504,11 +505,6 @@ class QuerySet(SearchableMixIn):
         new_qs._depth = depth
         return new_qs
 
-    def iterator(self):
-        # Return an iterator over the results
-        warnings.warn('QuerySet no longer caches results. .iterator() is a no-op.', DeprecationWarning, stacklevel=2)
-        return self.__iter__()
-
     ###########################
     #
     # Methods that end chaining
@@ -517,12 +513,12 @@ class QuerySet(SearchableMixIn):
 
     def get(self, *args, **kwargs):
         """Assume the query will return exactly one item. Return that item."""
-        if not args and set(kwargs) in ({'id'}, {'id', 'changekey'}):
+        if not args and set(kwargs) in ({"id"}, {"id", "changekey"}):
             # We allow calling get(id=..., changekey=...) to get a single item, but only if exactly these two
             # kwargs are present.
             account = self.folder_collection.account
-            item_id = self._id_field.field.clean(kwargs['id'], version=account.version)
-            changekey = self._changekey_field.field.clean(kwargs.get('changekey'), version=account.version)
+            item_id = self._id_field.field.clean(kwargs["id"], version=account.version)
+            changekey = self._changekey_field.field.clean(kwargs.get("changekey"), version=account.version)
             items = list(account.fetch(ids=[(item_id, changekey)], only_fields=self.only_fields))
         else:
             new_qs = self.filter(*args, **kwargs)
@@ -534,16 +530,17 @@ class QuerySet(SearchableMixIn):
         return items[0]
 
     def count(self, page_size=1000):
-        """Get the query count, with as little effort as possible 'page_size' is the number of items to
-        fetch from the server per request. We're only fetching the IDs, so keep it high
+        """Get the query count, with as little effort as possible
 
-        :param page_size:  (Default value = 1000)
+        :param page_size: The number of items to fetch per request. We're only fetching the IDs, so keep it high.
+        (Default value = 1000)
         """
         new_qs = self._copy_self()
         new_qs.only_fields = ()
         new_qs.order_fields = None
         new_qs.return_format = self.NONE
         new_qs.page_size = page_size
+        # 'chunk_size' not needed since we never need to call GetItem
         return len(list(new_qs.__iter__()))
 
     def exists(self):
@@ -559,101 +556,96 @@ class QuerySet(SearchableMixIn):
         new_qs.return_format = self.NONE
         return new_qs
 
-    def delete(self, page_size=1000, **delete_kwargs):
-        """Delete the items matching the query, with as little effort as possible. 'page_size' is the number of items
-        to fetch and delete per request. We're only fetching the IDs, so keep it high.
+    def delete(self, page_size=1000, chunk_size=100, **delete_kwargs):
+        """Delete the items matching the query, with as little effort as possible
 
-        :param page_size:  (Default value = 1000)
+        :param page_size: The number of items to fetch per request. We're only fetching the IDs, so keep it high.
+        (Default value = 1000)
+        :param chunk_size: The number of items to delete per request. (Default value = 100)
         :param delete_kwargs:
         """
         ids = self._id_only_copy_self()
         ids.page_size = page_size
-        return self.folder_collection.account.bulk_delete(
-            ids=ids,
-            chunk_size=page_size,
-            **delete_kwargs
-        )
+        return self.folder_collection.account.bulk_delete(ids=ids, chunk_size=chunk_size, **delete_kwargs)
 
-    def send(self, page_size=1000, **send_kwargs):
-        """Send the items matching the query, with as little effort as possible. 'page_size' is the number of items
-        to fetch and send per request. We're only fetching the IDs, so keep it high.
+    def send(self, page_size=1000, chunk_size=100, **send_kwargs):
+        """Send the items matching the query, with as little effort as possible
 
-        :param page_size:  (Default value = 1000)
+        :param page_size: The number of items to fetch per request. We're only fetching the IDs, so keep it high.
+        (Default value = 1000)
+        :param chunk_size: The number of items to send per request. (Default value = 100)
         :param send_kwargs:
         """
         ids = self._id_only_copy_self()
         ids.page_size = page_size
-        return self.folder_collection.account.bulk_send(
-            ids=ids,
-            chunk_size=page_size,
-            **send_kwargs
-        )
+        return self.folder_collection.account.bulk_send(ids=ids, chunk_size=chunk_size, **send_kwargs)
 
-    def copy(self, to_folder, page_size=1000, **copy_kwargs):
-        """Copy the items matching the query, with as little effort as possible. 'page_size' is the number of items
-        to fetch and copy per request. We're only fetching the IDs, so keep it high.
+    def copy(self, to_folder, page_size=1000, chunk_size=100, **copy_kwargs):
+        """Copy the items matching the query, with as little effort as possible
 
         :param to_folder:
-        :param page_size:  (Default value = 1000)
+        :param page_size: The number of items to fetch per request. We're only fetching the IDs, so keep it high.
+        (Default value = 1000)
+        :param chunk_size: The number of items to copy per request. (Default value = 100)
         :param copy_kwargs:
         """
         ids = self._id_only_copy_self()
         ids.page_size = page_size
         return self.folder_collection.account.bulk_copy(
-            ids=ids,
-            to_folder=to_folder,
-            chunk_size=page_size,
-            **copy_kwargs
+            ids=ids, to_folder=to_folder, chunk_size=chunk_size, **copy_kwargs
         )
 
-    def move(self, to_folder, page_size=1000):
+    def move(self, to_folder, page_size=1000, chunk_size=100):
         """Move the items matching the query, with as little effort as possible. 'page_size' is the number of items
         to fetch and move per request. We're only fetching the IDs, so keep it high.
 
         :param to_folder:
-        :param page_size: (Default value = 1000)
+        :param page_size: The number of items to fetch per request. We're only fetching the IDs, so keep it high.
+        (Default value = 1000)
+        :param chunk_size: The number of items to move per request. (Default value = 100)
         """
         ids = self._id_only_copy_self()
         ids.page_size = page_size
         return self.folder_collection.account.bulk_move(
             ids=ids,
             to_folder=to_folder,
-            chunk_size=page_size,
+            chunk_size=chunk_size,
         )
 
-    def archive(self, to_folder, page_size=1000):
+    def archive(self, to_folder, page_size=1000, chunk_size=100):
         """Archive the items matching the query, with as little effort as possible. 'page_size' is the number of items
         to fetch and move per request. We're only fetching the IDs, so keep it high.
 
         :param to_folder:
-        :param page_size: (Default value = 1000)
+        :param page_size: The number of items to fetch per request. We're only fetching the IDs, so keep it high.
+        (Default value = 1000)
+        :param chunk_size: The number of items to archive per request. (Default value = 100)
         """
         ids = self._id_only_copy_self()
         ids.page_size = page_size
         return self.folder_collection.account.bulk_archive(
             ids=ids,
             to_folder=to_folder,
-            chunk_size=page_size,
+            chunk_size=chunk_size,
         )
 
-    def mark_as_junk(self, page_size=1000, **mark_as_junk_kwargs):
+    def mark_as_junk(self, page_size=1000, chunk_size=1000, **mark_as_junk_kwargs):
         """Mark the items matching the query as junk, with as little effort as possible. 'page_size' is the number of
         items to fetch and mark per request. We're only fetching the IDs, so keep it high.
 
-        :param page_size:  (Default value = 1000)
+        :param page_size: The number of items to fetch per request. We're only fetching the IDs, so keep it high.
+        (Default value = 1000)
+        :param chunk_size: The number of items to mark as junk per request. (Default value = 100)
         :param mark_as_junk_kwargs:
         """
         ids = self._id_only_copy_self()
         ids.page_size = page_size
-        return self.folder_collection.account.bulk_mark_as_junk(
-            ids=ids,
-            chunk_size=page_size,
-            **mark_as_junk_kwargs
-        )
+        return self.folder_collection.account.bulk_mark_as_junk(ids=ids, chunk_size=chunk_size, **mark_as_junk_kwargs)
 
     def __str__(self):
-        fmt_args = [('q', str(self.q)), ('folders', '[%s]' % ', '.join(str(f) for f in self.folder_collection.folders))]
-        return self.__class__.__name__ + '(%s)' % ', '.join('%s=%s' % (k, v) for k, v in fmt_args)
+        fmt_args = [("q", str(self.q)), ("folders", f"[{', '.join(str(f) for f in self.folder_collection.folders)}]")]
+        args_str = ", ".join(f"{k}={v}" for k, v in fmt_args)
+        return f"{self.__class__.__name__}({args_str})"
 
 
 def _get_value_or_default(field, item):
