@@ -1,64 +1,77 @@
 import logging
-from collections import OrderedDict
 from copy import copy
 
-from .fields import InvalidField, FieldPath, DateTimeBackedDateField
-from .util import create_element, xml_to_str, value_to_xml_text, is_iterable
+from .errors import InvalidEnumValue
+from .fields import DateTimeBackedDateField, FieldPath, InvalidField
+from .util import create_element, is_iterable, value_to_xml_text, xml_to_str
 from .version import EXCHANGE_2010
 
 log = logging.getLogger(__name__)
 
 
 class Q:
-    """A class with an API similar to Django Q objects. Used to implemnt advanced filtering logic."""
+    """A class with an API similar to Django Q objects. Used to implement advanced filtering logic."""
 
     # Connection types
-    AND = 'AND'
-    OR = 'OR'
-    NOT = 'NOT'
-    NEVER = 'NEVER'  # This is not specified by EWS. We use it for queries that will never match, e.g. 'foo__in=()'
+    AND = "AND"
+    OR = "OR"
+    NOT = "NOT"
+    NEVER = "NEVER"  # This is not specified by EWS. We use it for queries that will never match, e.g. 'foo__in=()'
     CONN_TYPES = {AND, OR, NOT, NEVER}
 
     # EWS Operators
-    EQ = '=='
-    NE = '!='
-    GT = '>'
-    GTE = '>='
-    LT = '<'
-    LTE = '<='
-    EXACT = 'exact'
-    IEXACT = 'iexact'
-    CONTAINS = 'contains'
-    ICONTAINS = 'icontains'
-    STARTSWITH = 'startswith'
-    ISTARTSWITH = 'istartswith'
-    EXISTS = 'exists'
+    EQ = "=="
+    NE = "!="
+    GT = ">"
+    GTE = ">="
+    LT = "<"
+    LTE = "<="
+    EXACT = "exact"
+    IEXACT = "iexact"
+    CONTAINS = "contains"
+    ICONTAINS = "icontains"
+    STARTSWITH = "startswith"
+    ISTARTSWITH = "istartswith"
+    EXISTS = "exists"
     OP_TYPES = {EQ, NE, GT, GTE, LT, LTE, EXACT, IEXACT, CONTAINS, ICONTAINS, STARTSWITH, ISTARTSWITH, EXISTS}
     CONTAINS_OPS = {EXACT, IEXACT, CONTAINS, ICONTAINS, STARTSWITH, ISTARTSWITH}
 
     # Valid lookups
-    LOOKUP_RANGE = 'range'
-    LOOKUP_IN = 'in'
-    LOOKUP_NOT = 'not'
-    LOOKUP_GT = 'gt'
-    LOOKUP_GTE = 'gte'
-    LOOKUP_LT = 'lt'
-    LOOKUP_LTE = 'lte'
-    LOOKUP_EXACT = 'exact'
-    LOOKUP_IEXACT = 'iexact'
-    LOOKUP_CONTAINS = 'contains'
-    LOOKUP_ICONTAINS = 'icontains'
-    LOOKUP_STARTSWITH = 'startswith'
-    LOOKUP_ISTARTSWITH = 'istartswith'
-    LOOKUP_EXISTS = 'exists'
-    LOOKUP_TYPES = {LOOKUP_RANGE, LOOKUP_IN, LOOKUP_NOT, LOOKUP_GT, LOOKUP_GTE, LOOKUP_LT, LOOKUP_LTE, LOOKUP_EXACT,
-                    LOOKUP_IEXACT, LOOKUP_CONTAINS, LOOKUP_ICONTAINS, LOOKUP_STARTSWITH, LOOKUP_ISTARTSWITH,
-                    LOOKUP_EXISTS}
+    LOOKUP_RANGE = "range"
+    LOOKUP_IN = "in"
+    LOOKUP_NOT = "not"
+    LOOKUP_GT = "gt"
+    LOOKUP_GTE = "gte"
+    LOOKUP_LT = "lt"
+    LOOKUP_LTE = "lte"
+    LOOKUP_EXACT = "exact"
+    LOOKUP_IEXACT = "iexact"
+    LOOKUP_CONTAINS = "contains"
+    LOOKUP_ICONTAINS = "icontains"
+    LOOKUP_STARTSWITH = "startswith"
+    LOOKUP_ISTARTSWITH = "istartswith"
+    LOOKUP_EXISTS = "exists"
+    LOOKUP_TYPES = {
+        LOOKUP_RANGE,
+        LOOKUP_IN,
+        LOOKUP_NOT,
+        LOOKUP_GT,
+        LOOKUP_GTE,
+        LOOKUP_LT,
+        LOOKUP_LTE,
+        LOOKUP_EXACT,
+        LOOKUP_IEXACT,
+        LOOKUP_CONTAINS,
+        LOOKUP_ICONTAINS,
+        LOOKUP_STARTSWITH,
+        LOOKUP_ISTARTSWITH,
+        LOOKUP_EXISTS,
+    }
 
-    __slots__ = 'conn_type', 'field_path', 'op', 'value', 'children', 'query_string'
+    __slots__ = "conn_type", "field_path", "op", "value", "children", "query_string"
 
     def __init__(self, *args, **kwargs):
-        self.conn_type = kwargs.pop('conn_type', self.AND)
+        self.conn_type = kwargs.pop("conn_type", self.AND)
 
         self.field_path = None  # Name of the field we want to filter on
         self.op = None
@@ -76,15 +89,13 @@ class Q:
         # Parse args which must now be Q objects
         for q in args:
             if not isinstance(q, self.__class__):
-                raise ValueError("Non-keyword arg %r must be a Q instance" % q)
+                raise TypeError(f"Non-keyword arg {q!r} must be of type {Q}")
         self.children.extend(args)
 
         # Parse keyword args and extract the filter
         is_single_kwarg = len(args) == 0 and len(kwargs) == 1
         for key, value in kwargs.items():
-            self.children.extend(
-                self._get_children_from_kwarg(key=key, value=value, is_single_kwarg=is_single_kwarg)
-            )
+            self.children.extend(self._get_children_from_kwarg(key=key, value=value, is_single_kwarg=is_single_kwarg))
 
         # Simplify this object
         self.reduce()
@@ -96,24 +107,24 @@ class Q:
         """Generate Q objects corresponding to a single keyword argument. Make this a leaf if there are no children to
         generate.
         """
-        key_parts = key.rsplit('__', 1)
+        key_parts = key.rsplit("__", 1)
         if len(key_parts) == 2 and key_parts[1] in self.LOOKUP_TYPES:
             # This is a kwarg with a lookup at the end
             field_path, lookup = key_parts
             if lookup == self.LOOKUP_EXISTS:
                 # value=True will fall through to further processing
                 if not value:
-                    return [~self.__class__(**{key: True})]
+                    return (~self.__class__(**{key: True}),)
 
             if lookup == self.LOOKUP_RANGE:
                 # EWS doesn't have a 'range' operator. Emulate 'foo__range=(1, 2)' as 'foo__gte=1 and foo__lte=2'
                 # (both values inclusive).
                 if len(value) != 2:
-                    raise ValueError("Value of lookup '%s' must have exactly 2 elements" % key)
-                return [
-                    self.__class__(**{'%s__gte' % field_path: value[0]}),
-                    self.__class__(**{'%s__lte' % field_path: value[1]}),
-                ]
+                    raise ValueError(f"Value of lookup {key!r} must have exactly 2 elements")
+                return (
+                    self.__class__(**{f"{field_path}__gte": value[0]}),
+                    self.__class__(**{f"{field_path}__lte": value[1]}),
+                )
 
             # Filtering on list types is a bit quirky. The only lookup type I have found to work is:
             #
@@ -129,19 +140,19 @@ class Q:
             # 'Contains' element which only supports matching on string elements, not arrays.
             #
             # Exact matching of categories (i.e. match ['a', 'b'] but not ['a', 'b', 'c']) could be implemented by
-            # post-processing items by fetch the categories field unconditionally and removing the items that don't
+            # post-processing items by fetching the categories field unconditionally and removing the items that don't
             # have an exact match.
             if lookup == self.LOOKUP_IN:
                 # EWS doesn't have an '__in' operator. Allow '__in' lookups on list and non-list field types,
                 # specifying a list value. We'll emulate it as a set of OR'ed exact matches.
                 if not is_iterable(value, generators_allowed=True):
-                    raise ValueError("Value for lookup %r must be a list" % key)
-                children = [self.__class__(**{field_path: v}) for v in value]
+                    raise TypeError(f"Value for lookup {key!r} must be of type {list}")
+                children = tuple(self.__class__(**{field_path: v}) for v in value)
                 if not children:
                     # This is an '__in' operator with an empty list as the value. We interpret it to mean "is foo
                     # contained in the empty set?" which is always false. Mark this Q object as such.
-                    return [self.__class__(conn_type=self.NEVER)]
-                return [self.__class__(*children, conn_type=self.OR)]
+                    return (self.__class__(conn_type=self.NEVER),)
+                return (self.__class__(*children, conn_type=self.OR),)
 
             if lookup == self.LOOKUP_CONTAINS and is_iterable(value, generators_allowed=True):
                 # A '__contains' lookup with an list as the value ony makes sense for list fields, since exact match
@@ -149,24 +160,24 @@ class Q:
                 #
                 # An empty list as value is allowed. We interpret it to mean "are all values in the empty set contained
                 # in foo?" which is always true.
-                children = [self.__class__(**{field_path: v}) for v in value]
-                return [self.__class__(*children, conn_type=self.AND)]
+                children = tuple(self.__class__(**{field_path: v}) for v in value)
+                return (self.__class__(*children, conn_type=self.AND),)
 
             try:
                 op = self._lookup_to_op(lookup)
             except KeyError:
-                raise ValueError("Lookup '%s' is not supported (called as '%s=%r')" % (lookup, key, value))
+                raise ValueError(f"Lookup {lookup!r} is not supported (called as '{key}={value!r}')")
         else:
             field_path, op = key, self.EQ
 
         if not is_single_kwarg:
-            return [self.__class__(**{key: value})]
+            return (self.__class__(**{key: value}),)
 
         # This is a single-kwarg Q object with a lookup that requires a single value. Make this a leaf
         self.field_path = field_path
         self.op = op
         self.value = value
-        return []
+        return ()
 
     def reduce(self):
         """Simplify this object, if possible."""
@@ -215,6 +226,7 @@ class Q:
         validating. There's no reason to replicate much of that here.
         """
         from .folders import Folder
+
         self.to_xml(folders=[Folder()], version=version, applies_to=Restriction.ITEMS)
 
     @classmethod
@@ -237,28 +249,28 @@ class Q:
     @classmethod
     def _conn_to_xml(cls, conn_type):
         xml_tag_map = {
-            cls.AND: 't:And',
-            cls.OR: 't:Or',
-            cls.NOT: 't:Not',
+            cls.AND: "t:And",
+            cls.OR: "t:Or",
+            cls.NOT: "t:Not",
         }
         return create_element(xml_tag_map[conn_type])
 
     @classmethod
     def _op_to_xml(cls, op):
         xml_tag_map = {
-            cls.EQ: 't:IsEqualTo',
-            cls.NE: 't:IsNotEqualTo',
-            cls.GTE: 't:IsGreaterThanOrEqualTo',
-            cls.LTE: 't:IsLessThanOrEqualTo',
-            cls.LT: 't:IsLessThan',
-            cls.GT: 't:IsGreaterThan',
-            cls.EXISTS: 't:Exists',
+            cls.EQ: "t:IsEqualTo",
+            cls.NE: "t:IsNotEqualTo",
+            cls.GTE: "t:IsGreaterThanOrEqualTo",
+            cls.LTE: "t:IsLessThanOrEqualTo",
+            cls.LT: "t:IsLessThan",
+            cls.GT: "t:IsGreaterThan",
+            cls.EXISTS: "t:Exists",
         }
         if op in xml_tag_map:
             return create_element(xml_tag_map[op])
         valid_ops = cls.EXACT, cls.IEXACT, cls.CONTAINS, cls.ICONTAINS, cls.STARTSWITH, cls.ISTARTSWITH
         if op not in valid_ops:
-            raise ValueError("'op' %s must be one of %s" % (op, valid_ops))
+            raise InvalidEnumValue("op", op, valid_ops)
 
         # For description of Contains attribute values, see
         #     https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/contains
@@ -282,24 +294,18 @@ class Q:
         #    https://en.wikipedia.org/wiki/Graphic_character#Spacing_and_non-spacing_characters
         # we shouldn't ignore them ('a' would match both 'a' and 'å', the latter having a non-spacing character).
         if op in {cls.EXACT, cls.IEXACT}:
-            match_mode = 'FullString'
+            match_mode = "FullString"
         elif op in (cls.CONTAINS, cls.ICONTAINS):
-            match_mode = 'Substring'
+            match_mode = "Substring"
         elif op in (cls.STARTSWITH, cls.ISTARTSWITH):
-            match_mode = 'Prefixed'
+            match_mode = "Prefixed"
         else:
-            raise ValueError('Unsupported op: %s' % op)
+            raise ValueError(f"Unsupported op: {op}")
         if op in (cls.IEXACT, cls.ICONTAINS, cls.ISTARTSWITH):
-            compare_mode = 'IgnoreCase'
+            compare_mode = "IgnoreCase"
         else:
-            compare_mode = 'Exact'
-        return create_element(
-            't:Contains',
-            attrs=OrderedDict([
-                ('ContainmentMode', match_mode),
-                ('ContainmentComparison', compare_mode),
-            ])
-        )
+            compare_mode = "Exact"
+        return create_element("t:Contains", attrs=dict(ContainmentMode=match_mode, ContainmentComparison=compare_mode))
 
     def is_leaf(self):
         return not self.children
@@ -320,33 +326,33 @@ class Q:
         if self.query_string:
             return self.query_string
         if self.is_leaf():
-            expr = '%s %s %r' % (self.field_path, self.op, self.value)
+            expr = f"{self.field_path} {self.op} {self.value!r}"
         else:
             # Sort children by field name so we get stable output (for easier testing). Children should never be empty.
-            expr = (' %s ' % (self.AND if self.conn_type == self.NOT else self.conn_type)).join(
-                (c.expr() if c.is_leaf() or c.conn_type == self.NOT else '(%s)' % c.expr())
-                for c in sorted(self.children, key=lambda i: i.field_path or '')
+            expr = f" {self.AND if self.conn_type == self.NOT else self.conn_type} ".join(
+                (c.expr() if c.is_leaf() or c.conn_type == self.NOT else f"({c.expr()})")
+                for c in sorted(self.children, key=lambda i: i.field_path or "")
             )
         if self.conn_type == self.NOT:
             # Add the NOT operator. Put children in parens if there is more than one child.
             if self.is_leaf() or len(self.children) == 1:
-                return self.conn_type + ' %s' % expr
-            return self.conn_type + ' (%s)' % expr
+                return self.conn_type + f" {expr}"
+            return self.conn_type + f" ({expr})"
         return expr
 
     def to_xml(self, folders, version, applies_to):
         if self.query_string:
             self._check_integrity()
             if version.build < EXCHANGE_2010:
-                raise NotImplementedError('QueryString filtering is only supported for Exchange 2010 servers and later')
-            elem = create_element('m:QueryString')
+                raise NotImplementedError("QueryString filtering is only supported for Exchange 2010 servers and later")
+            elem = create_element("m:QueryString")
             elem.text = self.query_string
             return elem
         # Translate this Q object to a valid Restriction XML tree
         elem = self.xml_elem(folders=folders, version=version, applies_to=applies_to)
         if elem is None:
             return None
-        restriction = create_element('m:Restriction')
+        restriction = create_element("m:Restriction")
         restriction.append(elem)
         return restriction
 
@@ -359,43 +365,42 @@ class Q:
             return
         if self.query_string:
             if any([self.field_path, self.op, self.value, self.children]):
-                raise ValueError('Query strings cannot be combined with other settings')
+                raise ValueError("Query strings cannot be combined with other settings")
             return
         if self.conn_type not in self.CONN_TYPES:
-            raise ValueError("'conn_type' %s must be one of %s" % (self.conn_type, self.CONN_TYPES))
+            raise InvalidEnumValue("conn_type", self.conn_type, self.CONN_TYPES)
         if not self.is_leaf():
             for q in self.children:
                 if q.query_string and len(self.children) > 1:
-                    raise ValueError(
-                        'A query string cannot be combined with other restrictions'
-                    )
+                    raise ValueError("A query string cannot be combined with other restrictions")
             return
         if not self.field_path:
             raise ValueError("'field_path' must be set")
         if self.op not in self.OP_TYPES:
-            raise ValueError("'op' %s must be one of %s" % (self.op, self.OP_TYPES))
+            raise InvalidEnumValue("op", self.op, self.OP_TYPES)
         if self.op == self.EXISTS and self.value is not True:
             raise ValueError("'value' must be True when operator is EXISTS")
         if self.value is None:
-            raise ValueError('Value for filter on field path "%s" cannot be None' % self.field_path)
+            raise ValueError(f"Value for filter on field path {self.field_path!r} cannot be None")
         if is_iterable(self.value, generators_allowed=True):
             raise ValueError(
-                'Value %r for filter on field path "%s" must be a single value' % (self.value, self.field_path)
+                f"Value {self.value!r} for filter on field path {self.field_path!r} must be a single value"
             )
 
     def _validate_field_path(self, field_path, folder, applies_to, version):
         from .indexed_properties import MultiFieldIndexedElement
+
         if applies_to == Restriction.FOLDERS:
             # This is a restriction on Folder fields
             folder.validate_field(field=field_path.field, version=version)
         else:
             folder.validate_item_field(field=field_path.field, version=version)
         if not field_path.field.is_searchable:
-            raise ValueError("EWS does not support filtering on field '%s'" % field_path.field.name)
+            raise ValueError(f"EWS does not support filtering on field {field_path.field.name!r}")
         if field_path.subfield and not field_path.subfield.is_searchable:
-            raise ValueError("EWS does not support filtering on subfield '%s'" % field_path.subfield.name)
+            raise ValueError(f"EWS does not support filtering on subfield {field_path.subfield.name!r}")
         if issubclass(field_path.field.value_cls, MultiFieldIndexedElement) and not field_path.subfield:
-            raise ValueError("Field path '%s' must contain a subfield" % self.field_path)
+            raise ValueError(f"Field path {self.field_path!r} must contain a subfield")
 
     def _get_field_path(self, folders, applies_to, version):
         # Convert the string field path to a real FieldPath object. The path is validated using the given folders.
@@ -412,7 +417,7 @@ class Q:
             self._validate_field_path(field_path=field_path, folder=folder, applies_to=applies_to, version=version)
             break
         else:
-            raise InvalidField("Unknown field path %r on folders %s" % (self.field_path, folders))
+            raise InvalidField(f"Unknown field path {self.field_path!r} on folders {folders}")
         return field_path
 
     def _get_clean_value(self, field_path, version):
@@ -429,6 +434,7 @@ class Q:
         # Recursively build an XML tree structure of this Q object. If this is an empty leaf (the equivalent of Q()),
         # return None.
         from .indexed_properties import SingleFieldIndexedElement
+
         # Don't check self.value just yet. We want to return error messages on the field path first, and then the value.
         # This is done in _get_field_path() and _get_clean_value(), respectively.
         self._check_integrity()
@@ -448,14 +454,12 @@ class Q:
                 # We need to convert to datetime
                 clean_value = field_path.field.date_to_datetime(clean_value)
             elem.append(field_path.to_xml())
-            constant = create_element('t:Constant')
             if self.op != self.EXISTS:
-                # Use .set() to not fill up the create_element() cache with unique values
-                constant.set('Value', value_to_xml_text(clean_value))
+                constant = create_element("t:Constant", attrs=dict(Value=value_to_xml_text(clean_value)))
                 if self.op in self.CONTAINS_OPS:
                     elem.append(constant)
                 else:
-                    uriorconst = create_element('t:FieldURIOrConstant')
+                    uriorconst = create_element("t:FieldURIOrConstant")
                     uriorconst.append(constant)
                     elem.append(uriorconst)
         elif len(self.children) == 1:
@@ -465,7 +469,7 @@ class Q:
             # We have multiple children. If conn_type is NOT, then group children with AND. We'll add the NOT later
             elem = self._conn_to_xml(self.AND if self.conn_type == self.NOT else self.conn_type)
             # Sort children by field name so we get stable output (for easier testing). Children should never be empty
-            for c in sorted(self.children, key=lambda i: i.field_path or ''):
+            for c in sorted(self.children, key=lambda i: i.field_path or ""):
                 elem.append(c.xml_elem(folders=folders, version=version, applies_to=applies_to))
         if elem is None:
             return None  # Should not be necessary, but play safe
@@ -517,16 +521,16 @@ class Q:
         return hash(repr(self))
 
     def __str__(self):
-        return self.expr() or 'Q()'
+        return self.expr() or "Q()"
 
     def __repr__(self):
         if self.is_leaf():
             if self.query_string:
-                return self.__class__.__name__ + '(%r)' % self.query_string
+                return self.__class__.__name__ + f"({self.query_string!r})"
             if self.is_never():
-                return self.__class__.__name__ + '(conn_type=%r)' % (self.conn_type)
-            return self.__class__.__name__ + '(%s %s %r)' % (self.field_path, self.op, self.value)
-        sorted_children = tuple(sorted(self.children, key=lambda i: i.field_path or ''))
+                return self.__class__.__name__ + f"(conn_type={self.conn_type!r})"
+            return self.__class__.__name__ + f"({self.field_path} {self.op} {self.value!r})"
+        sorted_children = tuple(sorted(self.children, key=lambda i: i.field_path or ""))
         if self.conn_type == self.NOT or len(self.children) > 1:
             return self.__class__.__name__ + repr((self.conn_type,) + sorted_children)
         return self.__class__.__name__ + repr(sorted_children)
@@ -536,21 +540,18 @@ class Restriction:
     """Implement an EWS Restriction type."""
 
     # The type of item the restriction applies to
-    FOLDERS = 'folders'
-    ITEMS = 'items'
+    FOLDERS = "folders"
+    ITEMS = "items"
     RESTRICTION_TYPES = (FOLDERS, ITEMS)
 
     def __init__(self, q, folders, applies_to):
-        if not isinstance(q, Q):
-            raise ValueError("'q' value %r must be a Q instance" % q)
+        """
+        :param q: A Q instance
+        :param folders: A list of BaseFolder instances
+        :param applies_to: A member of the RESTRICTION_TYPES eum
+        """
         if q.is_empty():
             raise ValueError("Q object must not be empty")
-        from .folders import BaseFolder
-        for folder in folders:
-            if not isinstance(folder, BaseFolder):
-                raise ValueError("'folder' value %r must be a Folder instance" % folder)
-        if applies_to not in self.RESTRICTION_TYPES:
-            raise ValueError("'applies_to' must be one of %s" % (self.RESTRICTION_TYPES,))
         self.q = q
         self.folders = folders
         self.applies_to = applies_to
