@@ -1,17 +1,35 @@
-from collections import namedtuple
 import pickle
+from collections import namedtuple
+from unittest.mock import patch
 
 from exchangelib.account import Account
 from exchangelib.attachments import FileAttachment
 from exchangelib.configuration import Configuration
-from exchangelib.credentials import Credentials, DELEGATE
-from exchangelib.errors import ErrorAccessDenied, ErrorFolderNotFound, UnauthorizedError
+from exchangelib.credentials import DELEGATE, Credentials, OAuth2Credentials
+from exchangelib.errors import (
+    ErrorAccessDenied,
+    ErrorDelegateNoUser,
+    ErrorFolderNotFound,
+    ErrorInvalidUserSid,
+    ErrorNotDelegate,
+    UnauthorizedError,
+    UnknownTimeZone,
+)
+from exchangelib.ewsdatetime import UTC
 from exchangelib.folders import Calendar
 from exchangelib.items import Message
-from exchangelib.properties import DelegateUser, UserId, DelegatePermissions
-from exchangelib.protocol import Protocol, FaultTolerance
-from exchangelib.services import GetDelegate
-from exchangelib.version import Version, EXCHANGE_2007_SP1
+from exchangelib.properties import (
+    DelegatePermissions,
+    DelegateUser,
+    MailTips,
+    OutOfOffice,
+    RecipientAddress,
+    SendingAs,
+    UserId,
+)
+from exchangelib.protocol import FaultTolerance, Protocol
+from exchangelib.services import GetDelegate, GetMailTips
+from exchangelib.version import EXCHANGE_2007_SP1, Version
 
 from .common import EWSTest
 
@@ -20,35 +38,68 @@ class AccountTest(EWSTest):
     """Test features of the Account object."""
 
     def test_magic(self):
-        self.account.fullname = 'John Doe'
+        self.account.fullname = "John Doe"
         self.assertIn(self.account.primary_smtp_address, str(self.account))
         self.assertIn(self.account.fullname, str(self.account))
 
     def test_validation(self):
         with self.assertRaises(ValueError) as e:
             # Must have valid email address
-            Account(primary_smtp_address='blah')
+            Account(primary_smtp_address="blah")
         self.assertEqual(str(e.exception), "primary_smtp_address 'blah' is not an email address")
         with self.assertRaises(AttributeError) as e:
             # Non-autodiscover requires a config
-            Account(primary_smtp_address='blah@example.com', autodiscover=False)
-        self.assertEqual(str(e.exception), 'non-autodiscover requires a config')
-        with self.assertRaises(ValueError) as e:
-            # access type must be one of ACCESS_TYPES
-            Account(primary_smtp_address='blah@example.com', access_type=123)
-        self.assertEqual(str(e.exception), "'access_type' 123 must be one of ('impersonation', 'delegate')")
+            Account(primary_smtp_address="blah@example.com", autodiscover=False)
+        self.assertEqual(str(e.exception), "non-autodiscover requires a config")
         with self.assertRaises(ValueError) as e:
+            Account(primary_smtp_address="blah@example.com", access_type=123)
+        self.assertEqual(str(e.exception), "'access_type' 123 must be one of ['delegate', 'impersonation']")
+        with self.assertRaises(TypeError) as e:
             # locale must be a string
-            Account(primary_smtp_address='blah@example.com', locale=123)
-        self.assertEqual(str(e.exception), "Expected 'locale' to be a string, got 123")
-        with self.assertRaises(ValueError) as e:
+            Account(primary_smtp_address="blah@example.com", locale=123)
+        self.assertEqual(str(e.exception), "'locale' 123 must be of type <class 'str'>")
+        with self.assertRaises(TypeError) as e:
             # default timezone must be an EWSTimeZone
-            Account(primary_smtp_address='blah@example.com', default_timezone=123)
-        self.assertEqual(str(e.exception), "Expected 'default_timezone' to be an EWSTimeZone, got 123")
-        with self.assertRaises(ValueError) as e:
+            Account(primary_smtp_address="blah@example.com", default_timezone=123)
+        self.assertEqual(
+            str(e.exception), "'default_timezone' 123 must be of type <class 'exchangelib.ewsdatetime.EWSTimeZone'>"
+        )
+        with self.assertRaises(TypeError) as e:
             # config must be a Configuration
-            Account(primary_smtp_address='blah@example.com', config=123)
-        self.assertEqual(str(e.exception), "Expected 'config' to be a Configuration, got 123")
+            Account(primary_smtp_address="blah@example.com", config=123)
+        self.assertEqual(
+            str(e.exception), "'config' 123 must be of type <class 'exchangelib.configuration.Configuration'>"
+        )
+
+    @patch("locale.getlocale", side_effect=ValueError())
+    def test_getlocale_failure(self, m):
+        a = Account(
+            primary_smtp_address=self.account.primary_smtp_address,
+            access_type=DELEGATE,
+            config=Configuration(
+                service_endpoint=self.account.protocol.service_endpoint,
+                credentials=Credentials("john@example.com", "WRONG_PASSWORD"),
+                version=self.account.version,
+                retry_policy=self.retry_policy,
+            ),
+            autodiscover=False,
+        )
+        self.assertEqual(a.locale, None)
+
+    @patch("tzlocal.get_localzone", side_effect=UnknownTimeZone(""))
+    def test_tzlocal_failure(self, m):
+        a = Account(
+            primary_smtp_address=self.account.primary_smtp_address,
+            access_type=DELEGATE,
+            config=Configuration(
+                service_endpoint=self.account.protocol.service_endpoint,
+                credentials=Credentials("john@example.com", "WRONG_PASSWORD"),
+                version=self.account.version,
+                retry_policy=self.retry_policy,
+            ),
+            autodiscover=False,
+        )
+        self.assertEqual(a.default_timezone, UTC)
 
     def test_get_default_folder(self):
         # Test a normal folder lookup with GetFolder
@@ -60,7 +111,7 @@ class AccountTest(EWSTest):
         class MockCalendar1(Calendar):
             @classmethod
             def get_distinguished(cls, root):
-                raise ErrorAccessDenied('foo')
+                raise ErrorAccessDenied("foo")
 
         # Test an indirect folder lookup with FindItems
         folder = self.account.root.get_default_folder(MockCalendar1)
@@ -71,7 +122,7 @@ class AccountTest(EWSTest):
         class MockCalendar2(Calendar):
             @classmethod
             def get_distinguished(cls, root):
-                raise ErrorFolderNotFound('foo')
+                raise ErrorFolderNotFound("foo")
 
         # Test using the one folder of this folder type
         with self.assertRaises(ErrorFolderNotFound):
@@ -90,8 +141,8 @@ class AccountTest(EWSTest):
 
     def test_pickle(self):
         # Test that we can pickle various objects
-        item = Message(folder=self.account.inbox, subject='XXX', categories=self.categories).save()
-        attachment = FileAttachment(name='pickle_me.txt', content=b'')
+        item = Message(folder=self.account.inbox, subject="XXX", categories=self.categories).save()
+        attachment = FileAttachment(name="pickle_me.txt", content=b"")
         for o in (
             FaultTolerance(max_wait=3600),
             self.account.protocol,
@@ -111,21 +162,70 @@ class AccountTest(EWSTest):
 
     def test_mail_tips(self):
         # Test that mail tips work
-        self.assertEqual(self.account.mail_tips.recipient_address, self.account.primary_smtp_address)
+        self.assertEqual(self.account.mail_tips.recipient_address.email_address, self.account.primary_smtp_address)
+        # recipients must not be empty
+        list(
+            GetMailTips(protocol=self.account.protocol).call(
+                sending_as=SendingAs(email_address=self.account.primary_smtp_address),
+                recipients=[],
+                mail_tips_requested="All",
+            )
+        )
+        xml = b"""\
+<?xml version="1.0" encoding="utf-8"?>
+<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">
+    <s:Body>
+        <m:GetMailTipsResponse ResponseClass="Success"
+                   xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types"
+                   xmlns:m="http://schemas.microsoft.com/exchange/services/2006/messages">
+            <m:ResponseCode>NoError</m:ResponseCode>
+            <m:ResponseMessages>
+                <m:MailTipsResponseMessageType ResponseClass="Success">
+                    <m:ResponseCode>NoError</m:ResponseCode>
+                    <m:MailTips>
+                        <t:RecipientAddress>
+                            <t:EmailAddress>user2@contoso.com</t:EmailAddress>
+                            <t:RoutingType>SMTP</t:RoutingType>
+                        </t:RecipientAddress>
+                        <t:OutOfOffice>
+                            <t:ReplyBody>
+                                <t:Message/>
+                            </t:ReplyBody>
+                        </t:OutOfOffice>
+                        <t:CustomMailTip>Hello World Mailtips</t:CustomMailTip>
+                    </m:MailTips>
+                </m:MailTipsResponseMessageType>
+            </m:ResponseMessages>
+        </m:GetMailTipsResponse>
+    </s:Body>
+</s:Envelope>"""
+        self.assertEqual(
+            list(GetMailTips(protocol=None).parse(xml)),
+            [
+                MailTips(
+                    recipient_address=RecipientAddress(email_address="user2@contoso.com"),
+                    out_of_office=OutOfOffice(),
+                    custom_mail_tip="Hello World Mailtips",
+                )
+            ],
+        )
 
     def test_delegate(self):
         # The test server does not have any delegate info. Test that account.delegates works, and mock to test parsing
         # of a non-empty response.
-        self.assertGreaterEqual(
-            len(self.account.delegates),
-            0
-        )
-        self.assertGreaterEqual(
-            len(list(GetDelegate(account=self.account).call(user_ids=['foo@example.com'], include_permissions=True))),
-            0
-        )
+        self.assertGreaterEqual(len(self.account.delegates), 0)
+        with self.assertRaises(ErrorInvalidUserSid):
+            list(GetDelegate(account=self.account).call(user_ids=[UserId(sid="XXX")], include_permissions=True))
+        with self.assertRaises(ErrorDelegateNoUser):
+            list(GetDelegate(account=self.account).call(user_ids=["foo@example.com"], include_permissions=True))
+        with self.assertRaises(ErrorNotDelegate):
+            list(
+                GetDelegate(account=self.account).call(
+                    user_ids=[self.account.primary_smtp_address], include_permissions=True
+                )
+            )
 
-        xml = b'''\
+        xml = b"""\
 <?xml version="1.0" encoding="utf-8"?>
 <s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">
   <s:Body>
@@ -154,13 +254,13 @@ class AccountTest(EWSTest):
       <m:DeliverMeetingRequests>DelegatesAndMe</m:DeliverMeetingRequests>
       </m:GetDelegateResponse>
   </s:Body>
-</s:Envelope>'''
+</s:Envelope>"""
 
-        MockTZ = namedtuple('EWSTimeZone', ['ms_id'])
-        MockAccount = namedtuple('Account', ['access_type', 'primary_smtp_address', 'default_timezone', 'protocol'])
-        MockProtocol = namedtuple('Protocol', ['version'])
+        MockTZ = namedtuple("EWSTimeZone", ["ms_id"])
+        MockAccount = namedtuple("Account", ["access_type", "primary_smtp_address", "default_timezone", "protocol"])
+        MockProtocol = namedtuple("Protocol", ["version"])
         p = MockProtocol(version=Version(build=EXCHANGE_2007_SP1))
-        a = MockAccount(DELEGATE, 'foo@example.com', MockTZ('XXX'), protocol=p)
+        a = MockAccount(DELEGATE, "foo@example.com", MockTZ("XXX"), protocol=p)
 
         ws = GetDelegate(account=a)
         delegates = list(ws.parse(xml))
@@ -168,19 +268,19 @@ class AccountTest(EWSTest):
             delegates,
             [
                 DelegateUser(
-                    user_id=UserId(sid='SOME_SID', primary_smtp_address='foo@example.com', display_name='Foo Bar'),
+                    user_id=UserId(sid="SOME_SID", primary_smtp_address="foo@example.com", display_name="Foo Bar"),
                     delegate_permissions=DelegatePermissions(
-                        calendar_folder_permission_level='Author',
-                        inbox_folder_permission_level='Reviewer',
-                        contacts_folder_permission_level='None',
-                        notes_folder_permission_level='None',
-                        journal_folder_permission_level='None',
-                        tasks_folder_permission_level='None',
+                        calendar_folder_permission_level="Author",
+                        inbox_folder_permission_level="Reviewer",
+                        contacts_folder_permission_level="None",
+                        notes_folder_permission_level="None",
+                        journal_folder_permission_level="None",
+                        tasks_folder_permission_level="None",
                     ),
                     receive_copies_of_meeting_messages=False,
                     view_private_items=True,
                 )
-            ]
+            ],
         )
 
     def test_login_failure_and_credentials_update(self):
@@ -190,13 +290,12 @@ class AccountTest(EWSTest):
             access_type=DELEGATE,
             config=Configuration(
                 service_endpoint=self.account.protocol.service_endpoint,
-                credentials=Credentials(self.account.protocol.credentials.username, 'WRONG_PASSWORD'),
+                credentials=Credentials("john@example.com", "WRONG_PASSWORD"),
                 version=self.account.version,
-                auth_type=self.account.protocol.auth_type,
                 retry_policy=self.retry_policy,
             ),
             autodiscover=False,
-            locale='da_DK',
+            locale="da_DK",
         )
 
         # Should fail when credentials are wrong, but UnauthorizedError is caught and retried. Mock the needed methods
@@ -208,7 +307,7 @@ class AccountTest(EWSTest):
 
             def raise_response_errors(self, response):
                 if response.status_code == 401:
-                    raise UnauthorizedError('Invalid credentials for %s' % response.url)
+                    raise UnauthorizedError(f"Invalid credentials for {response.url}")
                 return super().raise_response_errors(response)
 
         try:
@@ -222,5 +321,39 @@ class AccountTest(EWSTest):
         with self.assertRaises(AttributeError):
             account.protocol.config.credentials = self.account.protocol.credentials
         # Should succeed after credentials update
+        account.protocol.config.auth_type = self.account.protocol.config.auth_type
         account.protocol.credentials = self.account.protocol.credentials
         account.root.refresh()
+
+    def test_protocol_default_values(self):
+        # Test that retry_policy and auth_type always get a value regardless of how we create an Account
+        a = Account(
+            self.account.primary_smtp_address,
+            autodiscover=False,
+            config=Configuration(
+                server=self.settings["server"],
+                credentials=self.account.protocol.credentials,
+            ),
+        )
+        self.assertIsNotNone(a.protocol.auth_type)
+        self.assertIsNotNone(a.protocol.retry_policy)
+
+        if isinstance(self.account.protocol.credentials, OAuth2Credentials):
+            self.skipTest("OAuth authentication does not work with POX autodiscover")
+
+        pox_credentials = Credentials(username=self.settings["username"], password=self.settings["password"])
+
+        a = Account(
+            self.settings["alias"],
+            autodiscover=True,
+            config=Configuration(
+                server=self.settings["server"],
+                credentials=pox_credentials,
+            ),
+        )
+        self.assertIsNotNone(a.protocol.auth_type)
+        self.assertIsNotNone(a.protocol.retry_policy)
+
+        a = Account(self.settings["alias"], autodiscover=True, credentials=pox_credentials)
+        self.assertIsNotNone(a.protocol.auth_type)
+        self.assertIsNotNone(a.protocol.retry_policy)
